package com.seikoinstruments.sdk.thermalprinter.sample;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Locale;

import com.google.android.gms.common.api.ApiException;
import com.google.android.gms.common.api.ResolvableApiException;
import com.google.android.gms.location.LocationRequest;
import com.google.android.gms.location.LocationServices;
import com.google.android.gms.location.LocationSettingsRequest;
import com.google.android.gms.location.LocationSettingsResponse;
import com.google.android.gms.location.LocationSettingsStatusCodes;
import com.google.android.gms.location.SettingsClient;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.seikoinstruments.sdk.thermalprinter.BarcodeScannerListener;
import com.seikoinstruments.sdk.thermalprinter.CallbackFunctionListener;
import com.seikoinstruments.sdk.thermalprinter.SmartLabelManager;
import com.seikoinstruments.sdk.thermalprinter.printerenum.*;
import com.seikoinstruments.sdk.thermalprinter.PrinterException;
import com.seikoinstruments.sdk.thermalprinter.PrinterManager;
import com.seikoinstruments.sdk.thermalprinter.sample.R;

import android.Manifest;
import android.app.FragmentTransaction;
import android.bluetooth.BluetoothAdapter;
import android.content.Context;
import android.content.Intent;
import android.content.IntentSender;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.content.pm.PackageManager;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.net.Uri;
import android.net.wifi.WifiManager;
import android.os.Build;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.preference.PreferenceManager;
import android.provider.Settings;
import android.support.annotation.NonNull;
import android.support.v4.app.ActivityCompat;
import android.support.v4.app.FragmentActivity;
import android.support.v4.content.PermissionChecker;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.Toast;


/**
 * Main Activity
 */
public class MainActivity extends FragmentActivity
        implements CallbackFunctionListener, BarcodeScannerListener {

    private static final int REQUEST_ENABLE_BLUETOOTH = 1001;
    private static final int REQUEST_SELECT_DEVICE = 1002;
    private static final int REQUEST_SETTING_PROPERTY = 1003;
    private static final int REQUEST_LOCATION_SETTING_RESOLUTION = 1004;
    private static final int REQUEST_CODE_WIFI = 1005;
    private static final int REQUEST_CODE_BLUETOOTH = 1006;


    // Dialog ID
    /** connect */
    protected static final int DIALOG_SELECT_PRINTER_MODEL = 1;
    /** sendText */
    protected static final int DIALOG_INPUT_TEXT = 2;
    /** sendBinary */
    protected static final int DIALOG_INPUT_BINARY = 3;
    /** getPrinterResponse */
    protected static final int DIALOG_SELECT_PRINTER_RESPONSE = 4;
    /** registerLogo(mobile) */
    protected static final int DIALOG_REGISTER_LOGO_ID1 = 5;
    /** registerLogo(pos) */
    protected static final int DIALOG_REGISTER_LOGO_ID2 = 6;
    /** unregisterLogo(mobile) */
    protected static final int DIALOG_UNREGISTER_LOGO_ID1 = 7;
    /** unregisterLogo(pos) */
    protected static final int DIALOG_UNREGISTER_LOGO_ID2 = 8;
    /** registerStyleSheet */
    protected static final int DIALOG_REGISTER_STYLE_SHEET_NO = 9;
    /** unregisterStyleSheet */
    protected static final int DIALOG_UNREGISTER_STYLE_SHEET_NO = 10;
    /** sendTextEx */
    protected static final int DIALOG_INPUT_TEXT_EX = 11;
    /** printBarcode */
    protected static final int DIALOG_PRINT_BARCODE = 12;
    /** printPDF417 */
    protected static final int DIALOG_PRINT_PDF417 = 13;
    /** printQRcode */
    protected static final int DIALOG_PRINT_QRCODE = 14;
    /** printLogo */
    protected static final int DIALOG_CUT_PAPER = 15;
    /** printLogo */
    protected static final int DIALOG_OPEN_DRAWER = 16;
    /** buzzer */
    protected static final int DIALOG_BUZZER = 17;
    /** externalBuzzer */
    protected static final int DIALOG_EXTERNAL_BUZZER = 18;
    /** printLogo(mobile) */
    protected static final int DIALOG_PRINT_LOGO1 = 19;
    /** printLogo(pos) */
    protected static final int DIALOG_PRINT_LOGO2 = 20;
    /** sendDataFile */
    protected static final int DIALOG_SEND_DATA_FILE = 21;
    /** printDataMatrix */
    protected static final int DIALOG_PRINT_DATAMATRIX = 22;
    /** printMaxiCode */
    protected static final int DIALOG_PRINT_MAXICODE = 23;
    /** printGS1DatabarStacked */
    protected static final int DIALOG_PRINT_GS1STACKED = 24;
    /** printGS1DatabarStackedOmnidirectional */
    protected static final int DIALOG_PRINT_GS1STACKEDOMNIDIRECTIONAL = 25;
    /** printGS1DatabarExpandedStacked */
    protected static final int DIALOG_PRINT_GS1EXPANDEDSTACKED = 26;
    /** showTemplate */
    protected static final int DIALOG_SHOW_TEMPLATE = 27;
    /** showSlide */
    protected static final int DIALOG_SHOW_SLIDE = 28;
    /** executeMacro */
    protected static final int DIALOG_EXECUTE_MACRO = 29;
    /** turnOnScreen */
    protected static final int DIALOG_TURN_ON_SCREEN = 30;
    /** selectTemplate */
    protected static final int DIALOG_SELECT_TEMPLATE = 31;
    /** setTemplateImageData */
    protected static final int DIALOG_SET_TEMPLATE_IMAGE_DATA = 32;
    /** selectTemplateTextObject */
    protected static final int DIALOG_SELECT_TEMPLATE_TEXT_OBJECT = 33;
    /** setTemplateTextAlignment */
    protected static final int DIALOG_SET_TEMPLATE_TEXT_ALIGNMENT = 34;
    /** setTemplateTextData */
    protected static final int DIALOG_SET_TEMPLATE_TEXT_DATA = 35;
    /** setTemplateTextLeftMargin */
    protected static final int DIALOG_SET_TEMPLATE_TEXT_LEFT_MARGIN = 36;
    /** setTemplateTextLineSpacing */
    protected static final int DIALOG_SET_TEMPLATE_TEXT_LINE_SPACING = 37;
    /** setTemplateTextBold */
    protected static final int DIALOG_SET_TEMPLATE_TEXT_BOLD = 38;
    /** setTemplateTextUnderline */
    protected static final int DIALOG_SET_TEMPLATE_TEXT_UNDERLINE = 39;
    /** setTemplateTextSize */
    protected static final int DIALOG_SET_TEMPLATE_TEXT_SIZE = 40;
    /** setTemplateTextFont */
    protected static final int DIALOG_SET_TEMPLATE_TEXT_FONT = 41;
    /** setTemplateTextRegisteredFont */
    protected static final int DIALOG_SET_TEMPLATE_TEXT_REGISTERED_FONT = 42;
    /** setTemplateTextRightSpacing */
    protected static final int DIALOG_SET_TEMPLATE_TEXT_RIGHT_SPACING = 43;
    /** setTemplateTextColor */
    protected static final int DIALOG_SET_TEMPLATE_TEXT_COLOR = 44;
    /** setTemplateBarcodeData */
    protected static final int DIALOG_SET_TEMPLATE_BARCODE_DATA = 45;
    /** setTemplateQrcodeData */
    protected static final int DIALOG_SET_TEMPLATE_QRCODE_DATA = 46;
    /** registerTemplate */
    protected static final int DIALOG_REGISTER_TEMPLATE = 47;
    /** unregisterTemplate */
    protected static final int DIALOG_UNREGISTER_TEMPLATE = 48;
    /** registerImageData */
    protected static final int DIALOG_REGISTER_IMAGE_DATA = 49;
    /** unregisterImageData */
    protected static final int DIALOG_UNREGISTER_IMAGE_DATA = 50;
    /** registerSlideData */
    protected static final int DIALOG_REGISTER_SLIDE_DATA = 51;
    /** unregisterSlideData */
    protected static final int DIALOG_UNREGISTER_SLIDE_DATA = 52;
    /** registerUserDefinedCharacter */
    protected static final int DIALOG_REGISTER_USER_DEFINED_CHARACTER = 53;
    /** registerOptionFont */
    protected static final int DIALOG_REGISTER_OPTION_FONT = 54;
    /** controlMacroRegistration */
    protected static final int DIALOG_CONTROL_MACRO_REGISTRATION = 55;
    /** getDisplayResponse */
    protected static final int DIALOG_GET_DISPLAY_RESPONSE = 56;
    /** setBarcodeScannerListener */
    protected static final int DIALOG_SET_BARCODE_SCANNER_LISTENER = 57;
    /** printAztecCode */
    protected static final int DIALOG_PRINT_AZTECCODE = 58;
    /** feedPosition */
    protected static final int DIALOG_FEED_POSITION = 59;
    /** selectSmartLabelFile */
    protected static final int DIALOG_SELECT_SMART_LABEL_FILE = 60;
    /** replaceSmartLabelTextData */
    protected static final int DIALOG_REPLACE_SMART_LABEL_TEXT_DATA = 61;
    /** replaceSmartLabelBarcodeData */
    protected static final int DIALOG_REPLACE_SMART_LABEL_BARCODE_DATA = 62;
    /** replaceSmartLabelImageData */
    protected static final int DIALOG_REPLACE_SMART_LABEL_IMAGE_DATA = 63;
    /** Finish Application */
    protected static final int DIALOG_FINISH_APP = 99;
    /** Bluetooth no support */
    protected static final int DIALOG_BLUETOOTH_NO_SUPPORT = 101;
    /** Enable Wifi */
    protected static final int DIALOG_ENABLE_WIFI = 102;

    /** Response request code */
    protected static final int RESPONSE_REQUEST_CODE = 0x0E;

    /** Template text color code */
    protected static final int TEMPLATE_TEXT_COLOR_BLACK = 0x000000;
    protected static final int TEMPLATE_TEXT_COLOR_BLUE = 0x0000FF;
    protected static final int TEMPLATE_TEXT_COLOR_GREEN = 0x00FF00;
    protected static final int TEMPLATE_TEXT_COLOR_CYAN = 0x00FFFF;
    protected static final int TEMPLATE_TEXT_COLOR_RED = 0xFF0000;
    protected static final int TEMPLATE_TEXT_COLOR_MAGENTA = 0xFF00FF;
    protected static final int TEMPLATE_TEXT_COLOR_YELLOW = 0xFFFF00;
    protected static final int TEMPLATE_TEXT_COLOR_WHITE = 0xFFFFFF;

    // Log message setting
    /** write "in" on log */
    protected static final boolean WRITE_LOG_IN = true;
    /** write "out" on log */
    protected static final boolean WRITE_LOG_OUT = false;

    /** PrinterManager SDK */
    private static PrinterManager mPrinterManager;

    /** SmartLabelManager SDK */
    private static SmartLabelManager mSmartLabelManager;

    /** Select port */
    private int mSelectPort = PrinterManager.PRINTER_TYPE_BLUETOOTH;

    /** Select file */
    private String mSelectPath = "";

    /** Set listener*/
    private static CallbackFunctionListener mCallbackFunctionListener = null;
    private static BarcodeScannerListener mBarcodeScannerListener = null;

    /** Barcode data display method*/
    private boolean mIsStringDisplay = false;

    /** Handler to write on log*/
    private static final Handler mHandler = new Handler();

    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_ALWAYS_HIDDEN);

        setContentView(R.layout.main);

        final EditText edtDeviceAddress = (EditText) findViewById(R.id.edittext_device_address);
        final Button btnDeviceList = (Button) findViewById(R.id.button_device_list);

        Object savedObject = getLastCustomNonConfigurationInstance();

        if(savedObject != null
                && savedObject instanceof ArrayList
                && ((ArrayList)savedObject).get(0) instanceof PrinterManager) {
            mPrinterManager = (PrinterManager) (((ArrayList) savedObject).get(0));
                edtDeviceAddress.setEnabled(true);
                btnDeviceList.setEnabled(true);
        } else {
            if(mPrinterManager == null){
                mPrinterManager = new PrinterManager(getApplicationContext());
            }
            SampleApplication application = (SampleApplication) this.getApplication();
            application.setPrinterManager(mPrinterManager);

            final SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(this);
            Editor editor = pref.edit();
            if (Locale.JAPAN.equals(Locale.getDefault())) {
                editor.putString(getString(R.string.key_international_character), getString(R.string.international_character_default_jp_values));
                editor.putString(getString(R.string.key_code_page), getString(R.string.code_page_default_jp_values));
            } else {
                editor.putString(getString(R.string.key_international_character), getString(R.string.international_character_default_en_values));
                editor.putString(getString(R.string.key_code_page), getString(R.string.code_page_default_en_values));
            }
            editor.commit();

        }

        // [Connection type]Radio Button
        RadioGroup radioGroup = (RadioGroup) findViewById(R.id.radiogroup);
        radioGroup.setOnCheckedChangeListener(new RadioGroup.OnCheckedChangeListener() {
            public void onCheckedChanged(RadioGroup group, int checkedId) {
                RadioButton radioButton = (RadioButton) findViewById(checkedId);
                String text = radioButton.getText().toString();

                if (text.equals(getString(R.string.usb))) {
                    mSelectPort = PrinterManager.PRINTER_TYPE_USB;
                } else if (text.equals(getString(R.string.tcpip))
                        || text.equals(getString(R.string.tcp))) {
                    mSelectPort = PrinterManager.PRINTER_TYPE_TCP;
                } else {
                    mSelectPort = PrinterManager.PRINTER_TYPE_BLUETOOTH;
                }
            }
        });

        // [List]Button
        if (btnDeviceList != null) {
            btnDeviceList.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    Intent intent = new Intent(MainActivity.this, DeviceActivity.class);
                    intent.putExtra(DeviceActivity.TYPE, mSelectPort);
                    if (mSelectPort == PrinterManager.PRINTER_TYPE_BLUETOOTH) {
                        if (!checkBluetooth()) {
                            return;
                        }
                        if (PermissionChecker.checkSelfPermission(MainActivity.this, Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
                            ActivityCompat.requestPermissions(MainActivity.this, new String[]{Manifest.permission.ACCESS_FINE_LOCATION}, REQUEST_CODE_BLUETOOTH);
                            return;
                        }

                        //for Android 12
                        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.S) {
                            if ((PermissionChecker.checkSelfPermission(MainActivity.this, Manifest.permission.BLUETOOTH_CONNECT) != PackageManager.PERMISSION_GRANTED)
                                    || (PermissionChecker.checkSelfPermission(MainActivity.this, Manifest.permission.BLUETOOTH_SCAN) != PackageManager.PERMISSION_GRANTED)) {
                                ActivityCompat.requestPermissions(MainActivity.this, new String[]{Manifest.permission.BLUETOOTH_CONNECT, Manifest.permission.BLUETOOTH_SCAN}, REQUEST_CODE_BLUETOOTH);
                                return;
                            }
                        }

                        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q) {
                            checkLocationRequest();
                            return;
                        }
                    } else if (mSelectPort == PrinterManager.PRINTER_TYPE_TCP) {
                        if (!checkWifi()) {
                            return;
                        }
                    }
                    startActivityForResult(intent, REQUEST_SELECT_DEVICE);
                }
            });
        }

        // [connect]Button
        Button btnConnect = (Button) findViewById(R.id.button_connect);
        if (btnConnect != null) {
            btnConnect.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_SELECT_PRINTER_MODEL);
                    }
                }
            });
        }

        // [disconnect]Button
        Button btnDisconnect = (Button) findViewById(R.id.button_disconnect);
        if (btnDisconnect != null) {
            btnDisconnect.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    writeLog(getString(R.string.disconnect), WRITE_LOG_IN);

                    int ret = 0;
                    String msg;
                    try {
                        mPrinterManager.disconnect();
                        msg = getString(R.string.disconnect) + getString(R.string.msg_ok);
                    } catch (PrinterException e) {
                        ret = e.getErrorCode();
                        msg = getString(R.string.disconnect) + getString(R.string.msg_ng, ret);
                    }
                    Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

                    writeLog(getString(R.string.disconnect), WRITE_LOG_OUT, ret);
                }
            });
        }

        // [sendText]Button
        Button btnSendText = (Button) findViewById(R.id.button_send_text);
        if (btnSendText != null) {
            btnSendText.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_INPUT_TEXT);
                    }
                }
            });
        }

        // [sendTextEx]Button
        Button btnSendTextEx = (Button) findViewById(R.id.button_send_text_ex);
        if (btnSendTextEx != null) {
            btnSendTextEx.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_INPUT_TEXT_EX);
                    }
                }
            });
        }

        // [printBarcode]Button
        Button btnPrintBarcode = (Button) findViewById(R.id.button_print_barcode);
        if (btnPrintBarcode != null) {
            btnPrintBarcode.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    showAlertDialog(DIALOG_PRINT_BARCODE);
                }
            });
        }

        // [printPDF417]Button
        Button btnPrintPdf417 = (Button) findViewById(R.id.button_print_pdf417);
        if (btnPrintPdf417 != null) {
            btnPrintPdf417.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_PRINT_PDF417);
                    }
                }
            });
        }

        // [printQRcode]Button
        Button btnPrintQrcode = (Button) findViewById(R.id.button_print_qrcode);
        if (btnPrintQrcode != null) {
            btnPrintQrcode.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_PRINT_QRCODE);
                    }
                }
            });
        }

        // [printDataMatrix]Button
        Button btnPrintDataMatrix = (Button) findViewById(R.id.button_print_datamatrix);
        if (btnPrintDataMatrix != null) {
            btnPrintDataMatrix.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_PRINT_DATAMATRIX);
                    }
                }
            });
        }

        // [printMaxiCode]Button
        Button btnPrintMaxiCode = (Button) findViewById(R.id.button_print_maxicode);
        if (btnPrintMaxiCode != null) {
            btnPrintMaxiCode.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_PRINT_MAXICODE);
                    }
                }
            });
        }

        // [printGS1DatabarStacked]Button
        Button btnPrintGS1Stacked = (Button) findViewById(R.id.button_print_gs1_stacked);
        if (btnPrintGS1Stacked != null) {
            btnPrintGS1Stacked.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_PRINT_GS1STACKED);
                    }
                }
            });
        }

        // [printGS1DatabarStacked]Button
        Button btnPrintGS1StackedOmnidirectional = (Button) findViewById(R.id.button_print_gs1_stackedomnidirectional);
        if (btnPrintGS1StackedOmnidirectional != null) {
            btnPrintGS1StackedOmnidirectional.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_PRINT_GS1STACKEDOMNIDIRECTIONAL);
                    }
                }
            });
        }

        // [printGS1DatabarStacked]Button
        Button btnPrintGS1ExpandedStacked = (Button) findViewById(R.id.button_print_gs1_expandedstacked);
        if (btnPrintGS1ExpandedStacked != null) {
            btnPrintGS1ExpandedStacked.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_PRINT_GS1EXPANDEDSTACKED);
                    }
                }
            });
        }

        // [printAztecCode]Button
        Button btnPrintAztecCode = (Button) findViewById(R.id.button_print_azteccode);
        if (btnPrintAztecCode != null) {
            btnPrintAztecCode.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_PRINT_AZTECCODE);
                    }
                }
            });
        }

        // [feedPosition]Button
        Button btnFeedPosition = (Button) findViewById(R.id.button_feed_position);
        if (btnFeedPosition != null) {
            btnFeedPosition.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_FEED_POSITION);
                    }
                }
            });
        }

        // [cutPaper]Button
        Button btnCutPaper = (Button) findViewById(R.id.button_cut_paper);
        if (btnCutPaper != null) {
            btnCutPaper.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_CUT_PAPER);
                    }
                }
            });
        }

        // [openDrawer]Button
        Button btnOpenDrawer = (Button) findViewById(R.id.button_print_open_drawer);
        if (btnOpenDrawer != null) {
            btnOpenDrawer.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_OPEN_DRAWER);
                    }
                }
            });
        }

        // [buzzer]Button
        Button btnBuzzer = (Button) findViewById(R.id.button_buzzer);
        if (btnBuzzer != null) {
            btnBuzzer.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_BUZZER);
                    }
                }
            });
        }

        // [externalBuzzer]Button
        Button btnExternalBuzzer = (Button) findViewById(R.id.button_external_buzzer);
        if (btnExternalBuzzer != null) {
            btnExternalBuzzer.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_EXTERNAL_BUZZER);
                    }
                }
            });
        }

        // [sendBinary]Button
        Button btnSendBinary = (Button) findViewById(R.id.button_send_binary);
        if (btnSendBinary != null) {
            btnSendBinary.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_INPUT_BINARY);
                    }
                }
            });
        }


        // [sendDataFile]Button
        Button btnSendDataFile = (Button) findViewById(R.id.button_send_data_file);
        if (btnSendDataFile != null) {
            btnSendDataFile.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showOpenFileDialog(DIALOG_SEND_DATA_FILE);
                    }
                }
            });
        }

        // [abort]Button
        Button btnAbort = (Button) findViewById(R.id.button_abort);
        if (btnAbort != null) {
            btnAbort.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    writeLog(getString(R.string.abort), WRITE_LOG_IN);

                    int ret = 0;
                    String msg;
                    try {
                        mPrinterManager.abort();

                        msg = getString(R.string.abort) + getString(R.string.msg_ok);
                    } catch (PrinterException e) {
                        ret = e.getErrorCode();
                        msg = getString(R.string.abort) + getString(R.string.msg_ng, ret);
                    }
                    Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

                    writeLog(getString(R.string.abort), WRITE_LOG_OUT, ret);
                }
            });
        }

        // [getStatus]Button
        Button btnGetStatus = (Button) findViewById(R.id.button_get_status);
        if (btnGetStatus != null) {
            btnGetStatus.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    writeLog(getString(R.string.get_status), WRITE_LOG_IN);

                    int[] buf = new int[1];
                    int ret = 0;
                    String msg;
                    try {
                        mPrinterManager.getStatus(buf);

                        msg = getString(R.string.get_status) + getString(R.string.msg_ok) + getString(R.string.msg_status, buf[0]);
                    } catch (PrinterException e) {
                        ret = e.getErrorCode();
                        msg = getString(R.string.get_status) + getString(R.string.msg_ng, ret);
                    }
                    Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

                    writeLog(getString(R.string.get_status), WRITE_LOG_OUT, ret, getString(R.string.msg_status, buf[0]));
                }
            });
        }

        // [getPrinterResponse]Button
        Button btnGetPrinterResponse = (Button) findViewById(R.id.button_get_printer_response);
        if (btnGetPrinterResponse != null) {
            btnGetPrinterResponse.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_SELECT_PRINTER_RESPONSE);
                    }
                }
            });
        }

        // [registerLogo]Button
        Button btnRegisterLogo = (Button) findViewById(R.id.button_register_logo);
        if (btnRegisterLogo != null) {
            btnRegisterLogo.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        /* Open file */
                        int printerModel = mPrinterManager.getPrinterModel();
                        if ((printerModel == PrinterManager.PRINTER_MODEL_DPU_S245)
                                || (printerModel == PrinterManager.PRINTER_MODEL_DPU_S445)) {
                            showOpenFileDialog(DIALOG_REGISTER_LOGO_ID1);
                        } else {
                            showOpenFileDialog(DIALOG_REGISTER_LOGO_ID2);
                        }
                    }
                }
            });
        }

        // [printLogo]Button
        Button btnPrintLogo = (Button) findViewById(R.id.button_print_logo);
        if (btnPrintLogo != null) {
            btnPrintLogo.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        int printerModel = mPrinterManager.getPrinterModel();
                        if ((printerModel == PrinterManager.PRINTER_MODEL_DPU_S245)
                                || (printerModel == PrinterManager.PRINTER_MODEL_DPU_S445)) {
                            showAlertDialog(DIALOG_PRINT_LOGO1);
                        } else {
                            showAlertDialog(DIALOG_PRINT_LOGO2);
                        }
                    }
                }
            });
        }

        // [unregisterLogo]Button
        Button btnUnregisterLogo = (Button) findViewById(R.id.button_unregister_logo);
        if (btnUnregisterLogo != null) {
            btnUnregisterLogo.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        int printerModel = mPrinterManager.getPrinterModel();
                        if ((printerModel == PrinterManager.PRINTER_MODEL_DPU_S245)
                                || (printerModel == PrinterManager.PRINTER_MODEL_DPU_S445)) {
                            showAlertDialog(DIALOG_UNREGISTER_LOGO_ID1);
                        } else {
                            showAlertDialog(DIALOG_UNREGISTER_LOGO_ID2);
                        }
                    }
                }
            });
        }

        // [registerStyleSheet]Button
        Button btnRegisterStyleSheet = (Button) findViewById(R.id.button_register_style_sheet);
        if (btnRegisterStyleSheet != null) {
            btnRegisterStyleSheet.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showOpenFileDialog(DIALOG_REGISTER_STYLE_SHEET_NO);
                    }
                }
            });
        }

        // [unregisterStyleSheet]Button
        Button btnUnregisterStyleSheet = (Button) findViewById(R.id.button_unregister_style_sheet);
        if (btnUnregisterStyleSheet != null) {
            btnUnregisterStyleSheet.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_UNREGISTER_STYLE_SHEET_NO);
                    }
                }
            });
        }

        // [cotrolTransaction]Button
        Button btnControlTransaction = (Button) findViewById(R.id.button_control_transaction);
        if (btnControlTransaction != null) {
            btnControlTransaction.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    writeLog(getString(R.string.control_transaction), WRITE_LOG_IN);
                    int ret = 0;
                    String msg;
                    try {
                        final String receiptTitle = " Receipt Sample \n\n";
                        final String receiptTime = "No.123456789   03/04/11 12:49 PM\n";
                        final String receiptContent =
                                "--------------------------------\n" +
                                        "GRILLED CHICKEN BREAST   $ 18.50\n" +
                                        "SIRLOIN STEAK            $ 32.00\n" +
                                        "ROAST LAMB               $ 20.00\n" +
                                        "SALAD                    $ 10.00\n" +
                                        "COKE                     $  3.50\n" +
                                        "COKE                     $  3.50\n" +
                                        "ICE CREAM                $  5.00\n" +
                                        "CHINESE NOODLE           $ 15.00\n" +
                                        "SUKIYAKI                 $ 30.00\n" +
                                        "SANDWICH                 $ 10.00\n" +
                                        "PIZZA                    $ 20.00\n" +
                                        "TEA                      $  3.50\n" +
                                        "COFFEE                   $  3.50\n\n" +
                                        "--------------------------------\n" +
                                        "        SUBTOTAL        $ 174.50\n" +
                                        "        SALES TAX       $   8.73\n" +
                                        "        TOTAL           $ 183.23\n\n" +
                                        "  Thank you and see you again!  \n";
                        final String feed = "\n\n";
                        final String receiptBarcode = "12345678905";
                        final String receipt2DCode = "ABC0123";
                        final int HEIGHT_BARCODE = 162;
                        final int ROW_2DCODE = 0;
                        final int COLUM_2DCODE = 0;
                        final int HEIGHT_2DCODE = 10;

                        mPrinterManager.controlTransaction(TransactionFunction.TRANSACTION_START);
                        if(mPrinterManager.getPrinterModel() != PrinterManager.PRINTER_MODEL_DPU_S245
                                && mPrinterManager.getPrinterModel() != PrinterManager.PRINTER_MODEL_DPU_S445){
                            mPrinterManager.sendTextEx(receiptTitle, CharacterBold.BOLD, CharacterUnderline.UNDERLINE_CANCEL, CharacterReverse.REVERSE_CANCEL, CharacterFont.FONT_A, CharacterScale.VARTICAL_1_HORIZONTAL_2, PrintAlignment.ALIGNMENT_CENTER);
                            mPrinterManager.sendTextEx(receiptTime + receiptContent, CharacterBold.BOLD_CANCEL, CharacterUnderline.UNDERLINE_CANCEL, CharacterReverse.REVERSE_CANCEL, CharacterFont.FONT_A, CharacterScale.VARTICAL_1_HORIZONTAL_1, PrintAlignment.ALIGNMENT_CENTER);
                            mPrinterManager.sendText(feed);
                            mPrinterManager.printBarcode(BarcodeSymbol.BARCODE_SYMBOL_UPC_A, receiptBarcode, ModuleSize.BARCODE_MODULE_WIDTH_3, HEIGHT_BARCODE, HriPosition.HRI_POSITION_BELOW, CharacterFont.FONT_A, PrintAlignment.ALIGNMENT_CENTER);
                            mPrinterManager.printPDF417(receipt2DCode, ErrorCorrection.PDF417_ERROR_CORRECTION_0, ROW_2DCODE, COLUM_2DCODE, ModuleSize.PDF417_MODULE_WIDTH_3, HEIGHT_2DCODE, PrintAlignment.ALIGNMENT_CENTER);
                            mPrinterManager.cutPaper(CuttingMethod.CUT_PARTIAL);
                        }else{
                            mPrinterManager.sendTextEx(receiptTitle, CharacterBold.BOLD, CharacterUnderline.UNDERLINE_CANCEL, CharacterFont.FONT_A, CharacterScale.VARTICAL_1_HORIZONTAL_2);
                            mPrinterManager.sendTextEx(receiptTime + receiptContent, CharacterBold.BOLD_CANCEL, CharacterUnderline.UNDERLINE_CANCEL, CharacterFont.FONT_A, CharacterScale.VARTICAL_1_HORIZONTAL_1);
                            mPrinterManager.sendText(feed);
                            mPrinterManager.printBarcode(BarcodeSymbol.BARCODE_SYMBOL_UPC_A, receiptBarcode, ModuleSize.BARCODE_MODULE_WIDTH_3, HEIGHT_BARCODE, HriPosition.HRI_POSITION_BELOW, CharacterFont.FONT_A, PrintAlignment.ALIGNMENT_CENTER);
                            mPrinterManager.printPDF417(receipt2DCode, ErrorCorrection.PDF417_ERROR_CORRECTION_0, ROW_2DCODE, COLUM_2DCODE, ModuleSize.PDF417_MODULE_WIDTH_3, HEIGHT_2DCODE, PrintAlignment.ALIGNMENT_CENTER);
                        }
                        mPrinterManager.controlTransaction(TransactionFunction.TRANSACTION_PRINT);
                        mPrinterManager.getPrinterResponse(PrinterManager.PRINTER_RESPONSE_REQUEST, new int[]{RESPONSE_REQUEST_CODE});
                        msg = getString(R.string.control_transaction) + getString(R.string.msg_ok);
                    } catch (PrinterException e) {
                        try{
                            mPrinterManager.controlTransaction(TransactionFunction.TRANSACTION_CLEAR);
                        }catch(PrinterException e1){
                        }
                        ret = e.getErrorCode();
                        msg = getString(R.string.control_transaction) + getString(R.string.msg_ng, ret);
                    }

                    Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();
                    writeLog(getString(R.string.control_transaction), WRITE_LOG_OUT, ret);
                }
            });
        }

        // [printPageMode]Button
        Button btnPrintPageMode = (Button) findViewById(R.id.button_print_page_mode);
        if (btnPrintPageMode != null) {
            btnPrintPageMode.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        doPrintPageMode();
                    }
                }
            });
        }

        // [setCallbackFunctionListener]Button
        Button btnSetCallbackFunctionListener = (Button) findViewById(R.id.button_set_status_callback_function_listener);
        if (btnSetCallbackFunctionListener != null) {
            btnSetCallbackFunctionListener.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    writeLog(getString(R.string.set_status_callback_function_listener), WRITE_LOG_IN);

                    int ret = 0;
                    String msg;
                    try{
                        mCallbackFunctionListener = MainActivity.this;
                        mPrinterManager.setCallbackFunctionListener(mCallbackFunctionListener);
                        msg = getString(R.string.set_status_callback_function_listener) + getString(R.string.msg_ok);
                    }catch(PrinterException e){
                        ret = e.getErrorCode();
                        msg = getString(R.string.set_status_callback_function_listener) + getString(R.string.msg_ng, ret);
                    }
                    Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

                    writeLog(getString(R.string.set_status_callback_function_listener), WRITE_LOG_OUT, ret);
                }
            });
        }

        // [setBarcodeScannerListener]Button
        Button btnBarcodeScannerListener = (Button) findViewById(R.id.button_set_barcode_scanner_listener);
        if (btnBarcodeScannerListener != null) {
            btnBarcodeScannerListener.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_SET_BARCODE_SCANNER_LISTENER);
                    }
                }
            });
        }

        // [resetPrinter]Button
        Button btnResetPrinter = (Button) findViewById(R.id.button_reset_printer);
        if (btnResetPrinter != null) {
            btnResetPrinter.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    writeLog(getString(R.string.reset_printer), WRITE_LOG_IN);

                    int ret = 0;
                    String msg;
                    try {
                        mPrinterManager.resetPrinter();

                        msg = getString(R.string.reset_printer) + getString(R.string.msg_ok);
                    } catch (PrinterException e) {
                        ret = e.getErrorCode();
                        msg = getString(R.string.reset_printer) + getString(R.string.msg_ng, ret);
                    }
                    Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

                    writeLog(getString(R.string.reset_printer), WRITE_LOG_OUT, ret);
                }
            });
        }

        // [showTemplate]Button
        Button btnShowTemplatePrinter = (Button) findViewById(R.id.button_show_template);
        if (btnShowTemplatePrinter != null) {
            btnShowTemplatePrinter.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_SHOW_TEMPLATE);
                    }
                }
            });
        }

        // [showSlide]Button
        Button btnShowSlide = (Button) findViewById(R.id.button_show_slide);
        if (btnShowSlide != null) {
            btnShowSlide.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_SHOW_SLIDE);
                    }
                }
            });
        }

        // [enterStandbyMode]Button
        Button btnEnterStandbyMode = (Button) findViewById(R.id.button_enter_standby_mode);
        if (btnEnterStandbyMode != null) {
            btnEnterStandbyMode.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    writeLog(getString(R.string.enter_standby_mode), WRITE_LOG_IN);

                    int ret = 0;
                    String msg;
                    try {
                        mPrinterManager.enterStandbyMode();

                        msg = getString(R.string.enter_standby_mode) + getString(R.string.msg_ok);
                    } catch (PrinterException e) {
                        ret = e.getErrorCode();
                        msg = getString(R.string.enter_standby_mode) + getString(R.string.msg_ng, ret);
                    }
                    Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

                    writeLog(getString(R.string.enter_standby_mode), WRITE_LOG_OUT, ret);
                }
            });
        }

        // [executeMacro]Button
        Button btnExecuteMacro = (Button) findViewById(R.id.button_execute_macro);
        if (btnExecuteMacro != null) {
            btnExecuteMacro.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_EXECUTE_MACRO);
                    }
                }
            });
        }

        // [turnOnScreen]Button
        Button btnTurnOnScreen = (Button) findViewById(R.id.button_turn_on_screen);
        if (btnTurnOnScreen != null) {
            btnTurnOnScreen.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_TURN_ON_SCREEN);
                    }
                }
            });
        }

        // [selectTemplate]Button
        Button btnSelectTemplate = (Button) findViewById(R.id.button_select_template);
        if (btnSelectTemplate != null) {
            btnSelectTemplate.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_SELECT_TEMPLATE);
                    }
                }
            });
        }

        // [setTemplateImageData]Button
        Button btnSetTemplateImageData = (Button) findViewById(R.id.button_set_template_image_data);
        if (btnSetTemplateImageData != null) {
            btnSetTemplateImageData.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_SET_TEMPLATE_IMAGE_DATA);
                    }
                }
            });
        }

        // [selectTemplateTextObject]Button
        Button btnSelectTemplateTextObject = (Button) findViewById(R.id.button_select_template_text_object);
        if (btnSelectTemplateTextObject != null) {
            btnSelectTemplateTextObject.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_SELECT_TEMPLATE_TEXT_OBJECT);
                    }
                }
            });
        }

        // [setTemplateTextAlignment]Button
        Button btnSetTemplateTextAlignmet = (Button) findViewById(R.id.button_set_template_text_alignment);
        if (btnSetTemplateTextAlignmet != null) {
            btnSetTemplateTextAlignmet.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_SET_TEMPLATE_TEXT_ALIGNMENT);
                    }
                }
            });
        }

        // [setTemplateTextData]Button
        Button btnSetTemplateTextData = (Button) findViewById(R.id.button_set_template_text_data);
        if (btnSetTemplateTextData != null) {
            btnSetTemplateTextData.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_SET_TEMPLATE_TEXT_DATA);
                    }
                }
            });
        }

        // [setTemplateTextLeftMargin]Button
        Button btnSetTemplateTextLeftMargin = (Button) findViewById(R.id.button_set_template_text_left_margin);
        if (btnSetTemplateTextLeftMargin != null) {
            btnSetTemplateTextLeftMargin.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_SET_TEMPLATE_TEXT_LEFT_MARGIN);
                    }
                }
            });
        }

        // [setTemplateTextLineSpacing]Button
        Button btnSetTemplateTextLineSpacing = (Button) findViewById(R.id.button_set_template_text_line_spacing);
        if (btnSetTemplateTextLineSpacing != null) {
            btnSetTemplateTextLineSpacing.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_SET_TEMPLATE_TEXT_LINE_SPACING);
                    }
                }
            });
        }

        // [setTemplateTextBold]Button
        Button btnSetTemplateTextBold = (Button) findViewById(R.id.button_set_template_text_bold);
        if (btnSetTemplateTextBold != null) {
            btnSetTemplateTextBold.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_SET_TEMPLATE_TEXT_BOLD);
                    }
                }
            });
        }

        // [setTemplateTextUnderline]Button
        Button btnSetTemplateTextUnderline = (Button) findViewById(R.id.button_set_template_text_underline);
        if (btnSetTemplateTextUnderline != null) {
            btnSetTemplateTextUnderline.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_SET_TEMPLATE_TEXT_UNDERLINE);
                    }
                }
            });
        }

        // [setTemplateTextSize]Button
        Button btnSetTemplateTextSize = (Button) findViewById(R.id.button_set_template_text_size);
        if (btnSetTemplateTextSize != null) {
            btnSetTemplateTextSize.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_SET_TEMPLATE_TEXT_SIZE);
                    }
                }
            });
        }

        // [setTemplateTextFont]Button
        Button btnSetTemplateTextFont = (Button) findViewById(R.id.button_set_template_text_font);
        if (btnSetTemplateTextFont != null) {
            btnSetTemplateTextFont.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_SET_TEMPLATE_TEXT_FONT);
                    }
                }
            });
        }

        // [setTemplateTextRegisteredFont]Button
        Button btnSetTemplateTextRegisteredFont = (Button) findViewById(R.id.button_set_template_text_registered_font);
        if (btnSetTemplateTextRegisteredFont != null) {
            btnSetTemplateTextRegisteredFont.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_SET_TEMPLATE_TEXT_REGISTERED_FONT);
                    }
                }
            });
        }

        // [setTemplateTextRightSpacing]Button
        Button btnSetTemplateTextRightSpacing = (Button) findViewById(R.id.button_set_template_text_right_spacing);
        if (btnSetTemplateTextRightSpacing != null) {
            btnSetTemplateTextRightSpacing.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_SET_TEMPLATE_TEXT_RIGHT_SPACING);
                    }
                }
            });
        }

        // [setTemplateTextColor]Button
        Button btnSetTemplateTextColor = (Button) findViewById(R.id.button_set_template_text_color);
        if (btnSetTemplateTextColor != null) {
            btnSetTemplateTextColor.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_SET_TEMPLATE_TEXT_COLOR);
                    }
                }
            });
        }

        // [setTemplateBarcodeData]Button
        Button btnSetTemplateBarcodeData = (Button) findViewById(R.id.button_set_template_barcode_data);
        if (btnSetTemplateBarcodeData != null) {
            btnSetTemplateBarcodeData.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_SET_TEMPLATE_BARCODE_DATA);
                    }
                }
            });
        }

        // [setTemplateQrCodeData]Button
        Button btnSetTemplateQrcodeData = (Button) findViewById(R.id.button_set_template_qrcode_data);
        if (btnSetTemplateQrcodeData != null) {
            btnSetTemplateQrcodeData.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_SET_TEMPLATE_QRCODE_DATA);
                    }
                }
            });
        }

        // [registerTemplate]Button
        Button btnRegisterTemplate = (Button) findViewById(R.id.button_register_template);
        if (btnRegisterTemplate != null) {
            btnRegisterTemplate.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showOpenFileDialog(DIALOG_REGISTER_TEMPLATE);
                    }
                }
            });
        }

        // [unregisterTemplate]Button
        Button btnUnregisterTemplate = (Button) findViewById(R.id.button_unregister_template);
        if (btnUnregisterTemplate != null) {
            btnUnregisterTemplate.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_UNREGISTER_TEMPLATE);
                    }
                }
            });
        }

        // [registerImageData]Button
        Button btnRegisterImageData = (Button) findViewById(R.id.button_register_image_data);
        if (btnRegisterImageData != null) {
            btnRegisterImageData.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showOpenFileDialog(DIALOG_REGISTER_IMAGE_DATA);
                    }
                }
            });
        }

        // [unregisterImageData]Button
        Button btnUnregisterImageData = (Button) findViewById(R.id.button_unregister_image_data);
        if (btnUnregisterImageData != null) {
            btnUnregisterImageData.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_UNREGISTER_IMAGE_DATA);
                    }
                }
            });
        }

        // [registerSlideData]Button
        Button btnRegisterSlideData = (Button) findViewById(R.id.button_register_slide_data);
        if (btnRegisterSlideData != null) {
            btnRegisterSlideData.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showOpenFileDialog(DIALOG_REGISTER_SLIDE_DATA);
                    }
                }
            });
        }

        // [unregisterSlideData]Button
        Button btnUnregisterSlideData = (Button) findViewById(R.id.button_unregister_slide_data);
        if (btnUnregisterSlideData != null) {
            btnUnregisterSlideData.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_UNREGISTER_SLIDE_DATA);
                    }
                }
            });
        }

        // [registerUserDefinedCharacter]Button
        Button btnRegisterUserDefinedCharacter = (Button) findViewById(R.id.button_register_user_defined_character);
        if (btnRegisterUserDefinedCharacter != null) {
            btnRegisterUserDefinedCharacter.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showOpenFileDialog(DIALOG_REGISTER_USER_DEFINED_CHARACTER );
                    }
                }
            });
        }

        // [unregisterUserDefinedCharacter]Button
        Button btnUnregisterUserDefinedCharacter = (Button) findViewById(R.id.button_unregister_user_defined_character);
        if (btnUnregisterUserDefinedCharacter != null) {
            btnUnregisterUserDefinedCharacter.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    writeLog(getString(R.string.unregister_user_defined_character), WRITE_LOG_IN);

                    int ret = 0;
                    String msg;
                    try {
                        mPrinterManager.unregisterUserDefinedCharacter();
                        msg = getString(R.string.unregister_user_defined_character) + getString(R.string.msg_ok);

                    } catch (PrinterException e) {
                        ret = e.getErrorCode();
                        msg = getString(R.string.unregister_user_defined_character) + getString(R.string.msg_ng, ret);

                    }
                    Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

                    writeLog(getString(R.string.unregister_user_defined_character), WRITE_LOG_OUT, ret);

                }
            });
        }

        // [registerOptionFont]Button
        Button btnRegisterOptionFont = (Button) findViewById(R.id.button_register_option_font);
        if (btnRegisterOptionFont != null) {
            btnRegisterOptionFont.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showOpenFileDialog(DIALOG_REGISTER_OPTION_FONT);
                    }
                }
            });
        }

        // [unregisterOptionFont]Button
        Button btnUnregisterOptionFont = (Button) findViewById(R.id.button_unregister_option_font);
        if (btnUnregisterOptionFont != null) {
            btnUnregisterOptionFont.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    writeLog(getString(R.string.unregister_option_font), WRITE_LOG_IN);

                    int ret = 0;
                    String msg;
                    try {
                        mPrinterManager.unregisterOptionFont();
                        msg = getString(R.string.unregister_option_font) + getString(R.string.msg_ok);

                    } catch (PrinterException e) {
                        ret = e.getErrorCode();
                        msg = getString(R.string.unregister_option_font) + getString(R.string.msg_ng, ret);

                    }
                    Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

                    writeLog(getString(R.string.unregister_option_font), WRITE_LOG_OUT, ret);

                }
            });
        }

        // [controlMacroRegistration]Button
        Button btnControlMacroRegistration = (Button) findViewById(R.id.button_control_macro_registration);
        if (btnControlMacroRegistration != null) {
            btnControlMacroRegistration.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_CONTROL_MACRO_REGISTRATION);
                    }
                }
            });
        }

        // [getDisplayResponse]Button
        Button btnGetDisplayResponse = (Button) findViewById(R.id.button_get_display_response);
        if (btnGetDisplayResponse != null) {
            btnGetDisplayResponse.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_GET_DISPLAY_RESPONSE);
                    }
                }
            });
        }

        // [defragment]Button
        Button btnDefragment = (Button) findViewById(R.id.button_defragment);
        if (btnDefragment != null) {
            btnDefragment.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    writeLog(getString(R.string.defragment), WRITE_LOG_IN);

                    int ret = 0;
                    String msg;
                    try {
                        mPrinterManager.defragment(MemoryArea.MEMORY_DISPLAY_USERMEMORY);
                        msg = getString(R.string.defragment) + getString(R.string.msg_ok);
                    } catch (PrinterException e) {
                        ret = e.getErrorCode();
                        msg = getString(R.string.defragment) + getString(R.string.msg_ng, ret);
                    }
                    Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

                    writeLog(getString(R.string.defragment), WRITE_LOG_OUT, ret);
                }
            });
        }

        // [printSmartLabelImageData]Button
        Button btnPrintSmartLabelImageData = (Button) findViewById(R.id.button_print_smart_label_image_data);
        if (btnPrintSmartLabelImageData != null) {
            btnPrintSmartLabelImageData.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        doPrintSmartLabelImageData();
                    }
                }
            });
        }

        // [selectSmartLabelFile]Button
        Button btnSelectSmartLabelFile = (Button) findViewById(R.id.button_select_smart_label_file);
        if (btnSelectSmartLabelFile != null) {
            btnSelectSmartLabelFile.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showOpenFileDialog(DIALOG_SELECT_SMART_LABEL_FILE);
                    }
                }
            });
        }

        // [replaceSmartLabelTextData]Button
        Button btnReplaceSmartLabelTextData = (Button) findViewById(R.id.button_replace_smart_label_text_data);
        if (btnReplaceSmartLabelTextData != null) {
            btnReplaceSmartLabelTextData.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_REPLACE_SMART_LABEL_TEXT_DATA);
                    }
                }
            });
        }

        // [replaceSmartLabelBarcodeData]Button
        Button btnReplaceSmartLabelBarcodeData = (Button) findViewById(R.id.button_replace_smart_label_barcode_data);
        if (btnReplaceSmartLabelBarcodeData != null) {
            btnReplaceSmartLabelBarcodeData.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showAlertDialog(DIALOG_REPLACE_SMART_LABEL_BARCODE_DATA);
                    }
                }
            });
        }

        // [replaceSmartLabelImageData]Button
        Button btnReplaceSmartLabelImageData = (Button) findViewById(R.id.button_replace_smart_label_image_data);
        if (btnReplaceSmartLabelImageData != null) {
            btnReplaceSmartLabelImageData.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {
                    if (!isFinishing()) {
                        showOpenFileDialog(DIALOG_REPLACE_SMART_LABEL_IMAGE_DATA);
                    }
                }
            });
        }
    }


    @Override
    public void onStart() {
        super.onStart();
    }


    @Override
    public void onResume() {
        super.onResume();
    }


    @Override
    public void onPause() {
        super.onPause();
    }


    @Override
    public void onRestart() {
        super.onRestart();
    }


    @Override
    public void onStop() {
        super.onStop();
    }


    @Override
    public void onDestroy() {
        super.onDestroy();
    }


    @Override
    public void onRequestPermissionsResult(int requestCode, String[] permissions, int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        if(requestCode ==REQUEST_CODE_BLUETOOTH){
            if (checkBluetooth()){

                //for Android 12
                if (grantResults.length == 1 ) {
                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.S) {
                        if ((PermissionChecker.checkSelfPermission(MainActivity.this, Manifest.permission.BLUETOOTH_CONNECT) != PackageManager.PERMISSION_GRANTED)
                                || (PermissionChecker.checkSelfPermission(MainActivity.this, Manifest.permission.BLUETOOTH_SCAN) != PackageManager.PERMISSION_GRANTED)) {
                            ActivityCompat.requestPermissions(MainActivity.this, new String[]{Manifest.permission.BLUETOOTH_CONNECT, Manifest.permission.BLUETOOTH_SCAN}, REQUEST_CODE_BLUETOOTH);
                            return;
                        }
                    }
                }

                //for Android 11
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q) {
                    checkLocationRequest();
                }else {
                    Intent intent = new Intent(MainActivity.this, DeviceActivity.class);
                    intent.putExtra(DeviceActivity.TYPE, mSelectPort);
                    startActivityForResult(intent, REQUEST_SELECT_DEVICE);
                }
            }
        }
    }

    @Override
    public void onBackPressed() {
        showAlertDialog(DIALOG_FINISH_APP);
    }


    protected void finishApp() {
        try {
            mPrinterManager.disconnect();
        } catch (PrinterException e) {
        }

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            finishAndRemoveTask();
        } else {
            super.finish();
        }
    }


    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.settings_menu, menu);

        return true;
    }


    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.item_settings:
                Intent intent = new Intent(MainActivity.this, SettingsActivity.class);
                startActivityForResult(intent, REQUEST_SETTING_PROPERTY);
                return true;
        }
        return false;
    }


    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        switch (requestCode) {
            case REQUEST_SELECT_DEVICE:
                if (resultCode == RESULT_OK) {
                    EditText edtDeviceAddress = (EditText) findViewById(R.id.edittext_device_address);
                    edtDeviceAddress.setText(data.getStringExtra(DeviceActivity.ADDRESS));
                }
                break;

            case REQUEST_SETTING_PROPERTY:
                setProperty();
                break;

            case REQUEST_LOCATION_SETTING_RESOLUTION:
                Intent intent = new Intent(MainActivity.this, DeviceActivity.class);
                intent.putExtra(DeviceActivity.TYPE, mSelectPort);
                startActivityForResult(intent, REQUEST_SELECT_DEVICE);
                break;

            // Choose a file
            case DIALOG_SEND_DATA_FILE:
            case DIALOG_REGISTER_LOGO_ID1:
            case DIALOG_REGISTER_LOGO_ID2:
            case DIALOG_REGISTER_STYLE_SHEET_NO:
            case DIALOG_REGISTER_TEMPLATE:
            case DIALOG_REGISTER_IMAGE_DATA:
            case DIALOG_REGISTER_SLIDE_DATA:
            case DIALOG_REGISTER_USER_DEFINED_CHARACTER:
            case DIALOG_REGISTER_OPTION_FONT:
            case DIALOG_SELECT_SMART_LABEL_FILE:
            case DIALOG_REPLACE_SMART_LABEL_IMAGE_DATA:
                if (resultCode == RESULT_OK) {
                    Uri uri = null;
                    if (data != null) {
                        uri = data.getData();
                        mSelectPath = uri != null?
                                uri.toString():
                                "";
                        onFileSelected(uri.toString(), requestCode);
                    }
                }
                break;
            default:
                break;
        }
    }


    @Override
    public Object onRetainCustomNonConfigurationInstance() {
        return (new ArrayList<PrinterManager>()).add(mPrinterManager);
    }


    @Override
    public void onStatusChanged(int status) {
        final int currentStatus = status;
        //statusChanged
        new Thread(new Runnable() {
            @Override
            public void run() {
                mHandler.post(new Runnable() {
                    @Override
                    public void run() {
                        writeLog(getString(R.string.on_status_changed), WRITE_LOG_IN, getString(R.string.msg_status, currentStatus));
                        writeLog(getString(R.string.on_status_changed), WRITE_LOG_OUT);
                    }
                });
            }
        }).start();
    }


    @Override
    public void onBarcodeScannerReadData(byte[] data) {
        String barcodeData;
        if (mIsStringDisplay){
            //String notation
            try {
                barcodeData = new String(data,"SJIS");
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
                barcodeData= "Encoding failed!!";
            }
        }else {
            //Hexadecimal notation
            StringBuilder sb = new StringBuilder(2 * data.length);
            for(byte b: data) {
                sb.append(String.format("%02x", b&0xff) );
            }
            barcodeData = sb.toString();
        }

        //BarcodeScanerReadData
        final String msg_barcode = barcodeData;
        new Thread(new Runnable() {
            @Override
            public void run() {
                mHandler.post(new Runnable() {
                    @Override
                    public void run() {
                        writeLog(getString(R.string.on_barcode_scanner_read_data), WRITE_LOG_IN, msg_barcode);
                        writeLog(getString(R.string.on_barcode_scanner_read_data), WRITE_LOG_OUT);
                    }
                });
            }
        }).start();
    }

    @Override
    public void onBarcodeScannerChangedOnline() {
        //BarcodeScanerChangedOnline
        new Thread(new Runnable() {
            @Override
            public void run() {
                mHandler.post(new Runnable() {
                    @Override
                    public void run() {
                        writeLog(getString(R.string.on_barcode_scanner_changed_online), WRITE_LOG_IN, getString(R.string.msg_online));
                        writeLog(getString(R.string.on_barcode_scanner_changed_online), WRITE_LOG_OUT);
                    }
                });
            }
        }).start();
    }

    @Override
    public void onBarcodeScannerChangedOffline() {
        //BarcodeScanerChangedOffline
        new Thread(new Runnable() {
            @Override
            public void run() {
                mHandler.post(new Runnable() {
                    @Override
                    public void run() {
                        writeLog(getString(R.string.on_barcode_scanner_changed_offline), WRITE_LOG_IN, getString(R.string.msg_offline));
                        writeLog(getString(R.string.on_barcode_scanner_changed_offline), WRITE_LOG_OUT);
                    }
                });
            }
        }).start();
    }

    /**
     * Implementation when pressing the [connect] button.
     *
     * @param which The element number of printer_model_values_list.
     */
    protected void doConnect(int which) {
        String[] printerModels = getResources().getStringArray(R.array.printer_model_values_list);
        int model = Integer.parseInt(printerModels[which]);
        EditText edtDeviceAddress = (EditText) findViewById(R.id.edittext_device_address);

        switch (mSelectPort) {
            case PrinterManager.PRINTER_TYPE_TCP:
                if (!checkWifi()) {
                    return;
                }
                break;

            default:
                if (!checkBluetooth()) {
                    return;
                }
        }

        writeLog(getString(R.string.connect), WRITE_LOG_IN, edtDeviceAddress.getText().toString());

        int ret = 0;
        String msg;
        try {
            setProperty();
            switch (mSelectPort) {
                case PrinterManager.PRINTER_TYPE_USB:
                    mPrinterManager.connect(model, edtDeviceAddress.getText().toString());
                    break;

                case PrinterManager.PRINTER_TYPE_TCP:
                    mPrinterManager.connect(model, edtDeviceAddress.getText().toString());
                    break;

                default:
                    final SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(MainActivity.this);
                    final boolean secure = pref.getBoolean(getString(R.string.key_secure_connection), true);
                    mPrinterManager.connect(model, edtDeviceAddress.getText().toString(), secure);
            }

            msg = getString(R.string.connect) + getString(R.string.msg_ok);
        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.connect) + getString(R.string.msg_ng, ret);
        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.connect), WRITE_LOG_OUT, ret);
    }


    /**
     * Implementation when pressing the [sendText] button.
     *
     * @param text Text data sent to a printer
     */
    protected void doSendText(String text) {
        writeLog(getString(R.string.send_text), WRITE_LOG_IN, text);

        text += getString(R.string.line_feed);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.sendText(text);

            msg = getString(R.string.send_text) + getString(R.string.msg_ok);
        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.send_text) + getString(R.string.msg_ng, ret);
        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.send_text), WRITE_LOG_OUT, ret);
    }


    /**
     * Implementation when pressing the [sendTextEx] button.
     *
     * @param text      Text data to send to a printer
     * @param bold      Bold print
     * @param underline Underline
     * @param reverse   Reverse print
     * @param font      Font
     * @param scale     Character scale
     * @param alignment Alignment
     */
    protected void doSendTextEx(String text, CharacterBold bold, CharacterUnderline underline, CharacterReverse reverse, CharacterFont font, CharacterScale scale, PrintAlignment alignment) {
        writeLog(getString(R.string.send_text_ex), WRITE_LOG_IN, text);

        text += getString(R.string.line_feed);

        int ret = 0;
        String msg;
        try {
            int printerModel = mPrinterManager.getPrinterModel();
            if ((printerModel == PrinterManager.PRINTER_MODEL_DPU_S245)
                    || (printerModel == PrinterManager.PRINTER_MODEL_DPU_S445)) {
                mPrinterManager.sendTextEx(
                        text,
                        bold,
                        underline,
                        font,
                        scale);
            } else {
                mPrinterManager.sendTextEx(
                        text,
                        bold,
                        underline,
                        reverse,
                        font,
                        scale,
                        alignment);
            }

            msg = getString(R.string.send_text_ex) + getString(R.string.msg_ok);
        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.send_text_ex) + getString(R.string.msg_ng, ret);
        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.send_text_ex), WRITE_LOG_OUT, ret);
    }

    /**
     * Implementation when pressing the [printBarcode] button.
     *
     * @param barcodeSymbol Barcode symbol
     * @param data          Barcode data to send to the printer
     * @param moduleSize    Barcode width
     * @param moduleHeight  Barcode height
     * @param hriPosition   HRI character print position
     * @param hriFont       HRI character font
     * @param alignment     Alignment
     * @param nwRatio       N:W ratio
     */
    protected void doPrintBarcode(BarcodeSymbol barcodeSymbol, String data, ModuleSize moduleSize, int moduleHeight, HriPosition hriPosition, CharacterFont hriFont, PrintAlignment alignment, NwRatio nwRatio) {
        writeLog(getString(R.string.print_barcode), WRITE_LOG_IN, data);

        int ret = 0;
        String msg;
        try {
            switch (barcodeSymbol) {
                case BARCODE_SYMBOL_UPC_A:
                case BARCODE_SYMBOL_UPC_E:
                case BARCODE_SYMBOL_EAN13:
                case BARCODE_SYMBOL_JAN13:
                case BARCODE_SYMBOL_EAN8:
                case BARCODE_SYMBOL_JAN8:
                case BARCODE_SYMBOL_EAN13_ADDON:
                case BARCODE_SYMBOL_JAN13_ADDON:
                case BARCODE_SYMBOL_GS1_OMNI_DIRECTIONAL:
                case BARCODE_SYMBOL_GS1_TRUNCATED:
                case BARCODE_SYMBOL_GS1_LIMITED:
                case BARCODE_SYMBOL_GS1_EXPANDED:
                    mPrinterManager.printBarcode(
                            barcodeSymbol,
                            data,
                            moduleSize,
                            moduleHeight,
                            hriPosition,
                            hriFont,
                            alignment
                    );
                    break;

                case BARCODE_SYMBOL_CODE39:
                case BARCODE_SYMBOL_ITF:
                case BARCODE_SYMBOL_CODABAR:
                    mPrinterManager.printBarcode(
                            barcodeSymbol,
                            data,
                            moduleSize,
                            moduleHeight,
                            hriPosition,
                            hriFont,
                            alignment,
                            nwRatio
                    );
                    break;
                case BARCODE_SYMBOL_CODE93:
                case BARCODE_SYMBOL_CODE128:
                    byte[] byteData = asByteArray(data);
                    mPrinterManager.printBarcode(
                            barcodeSymbol,
                            byteData,
                            moduleSize,
                            moduleHeight,
                            hriPosition,
                            hriFont,
                            alignment
                    );
                    break;
            }

            msg = getString(R.string.print_barcode) + getString(R.string.msg_ok);
        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.print_barcode) + getString(R.string.msg_ng, ret);
        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.print_barcode), WRITE_LOG_OUT, ret);
    }


    /**
     * Implementation when pressing the [printPdf417] button.
     *
     * @param data            Barcode data to send to the printer
     * @param errorCorrection Error correction level
     * @param row             The number of rows
     * @param column          The number of columns in data area
     * @param moduleSize      Nominal fine element width
     * @param moduleHeight    Module height
     * @param alignment       Alignment
     * @param pdf417Symbol    Symbol of PDF417
     */
    protected void doPrintPdf417(String data, ErrorCorrection errorCorrection, int row, int column, ModuleSize moduleSize, int moduleHeight, PrintAlignment alignment, Pdf417Symbol pdf417Symbol) {
        writeLog(getString(R.string.print_pdf417), WRITE_LOG_IN, data);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.printPDF417(
                    data,
                    errorCorrection,
                    row,
                    column,
                    moduleSize,
                    moduleHeight,
                    alignment,
                    pdf417Symbol
            );

            msg = getString(R.string.print_pdf417) + getString(R.string.msg_ok);
        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.print_pdf417) + getString(R.string.msg_ng, ret);
        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.print_pdf417), WRITE_LOG_OUT, ret);
    }


    /**
     * Implementation when pressing the [printQrcode] button.
     *
     * @param data            Barcode data to send to the printer
     * @param errorCorrection Error correction level
     * @param moduleSize      Module size
     * @param alignment       Alignment
     * @param model           QR Code Model
     */
    protected void doPrintQrcode(String data, ErrorCorrection errorCorrection, ModuleSize moduleSize, PrintAlignment alignment, QrModel model) {
        writeLog(getString(R.string.print_qrcode), WRITE_LOG_IN, data);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.printQRcode(
                    data,
                    errorCorrection,
                    moduleSize,
                    alignment,
                    model
            );

            msg = getString(R.string.print_qrcode) + getString(R.string.msg_ok);
        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.print_qrcode) + getString(R.string.msg_ng, ret);
        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.print_qrcode), WRITE_LOG_OUT, ret);
    }


    /**
     * Implementation when pressing the [printDataMatrix] button.
     *
     * @param data             Barcode data to send to the printer
     * @param dataMatrixModule The number of Data Matrix modules
     * @param moduleSize       Module size
     * @param alignment        Alignment
     */
    protected void doPrintDataMatrix(String data, DataMatrixModule dataMatrixModule, ModuleSize moduleSize, PrintAlignment alignment) {
        writeLog(getString(R.string.print_datamatrix), WRITE_LOG_IN, data);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.printDataMatrix(
                    data,
                    dataMatrixModule,
                    moduleSize,
                    alignment
            );

            msg = getString(R.string.print_datamatrix) + getString(R.string.msg_ok);
        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.print_datamatrix) + getString(R.string.msg_ng, ret);
        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.print_datamatrix), WRITE_LOG_OUT, ret);
    }


    /**
     * Implementation when pressing the [printMaxiCode] button.
     *
     * @param data         Barcode data to send to the printer
     * @param maxiCodeMode MaxiCode mode
     * @param alignment    Alignment
     */
    protected void doPrintMaxiCode(String data, MaxiCodeMode maxiCodeMode, PrintAlignment alignment) {
        writeLog(getString(R.string.print_maxicode), WRITE_LOG_IN, data);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.printMaxiCode(
                    data,
                    maxiCodeMode,
                    alignment
            );

            msg = getString(R.string.print_maxicode) + getString(R.string.msg_ok);
        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.print_maxicode) + getString(R.string.msg_ng, ret);
        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.print_maxicode), WRITE_LOG_OUT, ret);
    }


    /**
     * Implementation when pressing the [printGS1DataBarStacked] button.
     *
     * @param data       Barcode data to send to the printer
     * @param moduleSize Module size
     * @param alignment  Alignment
     */
    protected void doPrintGs1Stacked(String data, ModuleSize moduleSize, PrintAlignment alignment) {
        writeLog(getString(R.string.print_gs1_stacked), WRITE_LOG_IN, data);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.printGS1DataBarStacked(
                    data,
                    moduleSize,
                    alignment
            );
            msg = getString(R.string.print_gs1_stacked) + getString(R.string.msg_ok);
        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.print_gs1_stacked) + getString(R.string.msg_ng, ret);
        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.print_gs1_stacked), WRITE_LOG_OUT, ret);
    }


    /**
     * Implementation when pressing the [printGS1DataBarStackedOmnidirectional] button.
     *
     * @param data         Barcode data to send to the printer
     * @param moduleHeight Barcode module height (the number of the modules)
     * @param moduleSize   Module size
     * @param alignment    Alignment
     */
    protected void doPrintGs1StackedOmnidirectional(String data, int moduleHeight, ModuleSize moduleSize, PrintAlignment alignment) {
        writeLog(getString(R.string.print_gs1_stackedomnidirectional), WRITE_LOG_IN, data);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.printGS1DataBarStackedOmnidirectional(
                    data,
                    moduleHeight,
                    moduleSize,
                    alignment
            );
            msg = getString(R.string.print_gs1_stackedomnidirectional) + getString(R.string.msg_ok);
        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.print_gs1_stackedomnidirectional) + getString(R.string.msg_ng, ret);
        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.print_gs1_stackedomnidirectional), WRITE_LOG_OUT, ret);
    }


    /**
     * Implementation when pressing the [printGS1DatabarExpandedStacked] button.
     *
     * @param data       Barcode data to send to the printer
     * @param column     The number of columns
     * @param moduleSize Module size
     * @param alignment  Alignment
     */
    protected void doPrintGs1ExpandedStacked(String data, int column, ModuleSize moduleSize, PrintAlignment alignment) {
        writeLog(getString(R.string.print_gs1_expandedstacked), WRITE_LOG_IN, data);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.printGS1DataBarExpandedStacked(
                    data,
                    column,
                    moduleSize,
                    alignment
            );
            msg = getString(R.string.print_gs1_expandedstacked) + getString(R.string.msg_ok);
        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.print_gs1_expandedstacked) + getString(R.string.msg_ng, ret);
        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.print_gs1_expandedstacked), WRITE_LOG_OUT, ret);
    }

    /**
     * Implementation when pressing the [printAztecCode] button.
     * @param data Barcode data to send to the printer
     * @param layer The number of layer
     * @param errorCorrection The number of errorCorrection
     * @param moduleSize Module size
     * @param aztecSymbol AztecSymbol
     * @param alignment Alignment
     */
    protected void doPrintAztecCode(String data, int layer, int errorCorrection, ModuleSize moduleSize, AztecSymbol aztecSymbol, PrintAlignment alignment) {
        writeLog(getString(R.string.print_azteccode), WRITE_LOG_IN, data);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.printAztecCode(
                    data,
                    layer,
                    errorCorrection,
                    moduleSize,
                    aztecSymbol,
                    alignment
            );
            msg = getString(R.string.print_azteccode) + getString(R.string.msg_ok);
        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.print_azteccode) + getString(R.string.msg_ng, ret);
        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.print_azteccode), WRITE_LOG_OUT, ret);
    }


    /**
     * Implementation when pressing the [feedPosition] button.
     *
     * @param feedPosition Feed Position
     * @param feedPositionItem   For log output
     */
    protected void doFeedPosition(FeedPosition feedPosition, String feedPositionItem) {
        writeLog(getString(R.string.feed_position), WRITE_LOG_IN, feedPositionItem);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.feedPosition(feedPosition);

            msg = getString(R.string.feed_position) + getString(R.string.msg_ok);
        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.feed_position) + getString(R.string.msg_ng, ret);
        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.feed_position), WRITE_LOG_OUT, ret);
    }


    /**
     * Implementation when pressing the [cutPaper] button.
     *
     * @param cuttingMethod Cutting method
     * @param cutTypeItem   For log output
     */
    protected void doCutPaper(CuttingMethod cuttingMethod, String cutTypeItem) {
        writeLog(getString(R.string.cut_paper), WRITE_LOG_IN, cutTypeItem);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.cutPaper(cuttingMethod);

            msg = getString(R.string.cut_paper) + getString(R.string.msg_ok);
        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.cut_paper) + getString(R.string.msg_ng, ret);
        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.cut_paper), WRITE_LOG_OUT, ret);
    }


    /**
     * Implementation when pressing the [openDrawer] button.
     *
     * @param drawerNum      Drawer number
     * @param pulseWidth     pulse width
     * @param drawerNumItem  For log output
     * @param pulseWidthItem For log output
     */
    protected void doOpenDrawer(DrawerNum drawerNum, PulseWidth pulseWidth, String drawerNumItem, String pulseWidthItem) {
        writeLog(getString(R.string.open_drawer), WRITE_LOG_IN, getString(R.string.msg_drawer, drawerNumItem, pulseWidthItem));

        int ret = 0;
        String msg;
        try {
            mPrinterManager.openDrawer(drawerNum, pulseWidth);

            msg = getString(R.string.open_drawer) + getString(R.string.msg_ok);
        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.open_drawer) + getString(R.string.msg_ng, ret);
        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.open_drawer), WRITE_LOG_OUT, ret);
    }


    /**
     * Implementation when pressing the [buzzer] button.
     *
     * @param onTime  Buzzer On time (millisecond)
     * @param offTime Buzzer Off time (millisecond)
     */
    protected void doBuzzer(int onTime, int offTime) {
        writeLog(getString(R.string.buzzer), WRITE_LOG_IN, getString(R.string.msg_buzzer, Integer.toString(onTime), Integer.toString(offTime)));

        int ret = 0;
        String msg;
        try {
            mPrinterManager.buzzer(onTime, offTime);

            msg = getString(R.string.buzzer) + getString(R.string.msg_ok);
        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.buzzer) + getString(R.string.msg_ng, ret);
        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.buzzer), WRITE_LOG_OUT, ret);
    }


    /**
     * Implementation when pressing the [externalBuzzer] button.
     *
     * @param buzzerPattern  Buzzer pattern
     * @param buzzerCount Buzzer count
     */
    protected void doExternalBuzzer(BuzzerPattern buzzerPattern, int buzzerCount) {
        writeLog(getString(R.string.externalBuzzer), WRITE_LOG_IN, getString(R.string.msg_buzzer, Integer.toString(buzzerPattern.getValue()), Integer.toString(buzzerCount)));

        int ret = 0;
        String msg;
        try {
            mPrinterManager.externalBuzzer(buzzerPattern, buzzerCount);

            msg = getString(R.string.externalBuzzer) + getString(R.string.msg_ok);
        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.externalBuzzer) + getString(R.string.msg_ng, ret);
        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.externalBuzzer), WRITE_LOG_OUT, ret);
    }


    /**
     * Implementation when pressing the [sendBinary] button.
     *
     * @param binaryText Binary data to send to the printer
     */
    protected void doSendBinary(String binaryText) {
        writeLog(getString(R.string.send_binary), WRITE_LOG_IN, binaryText);

        int ret = 0;
        String msg;
        byte[] binary = asByteArray(binaryText);
        try {
            mPrinterManager.sendBinary(binary);
            msg = getString(R.string.send_binary) + getString(R.string.msg_ok);
        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.send_binary) + getString(R.string.msg_ng, ret);
        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.send_binary), WRITE_LOG_OUT, ret);

    }


    /**
     * Implementation when pressing the [sendDataFile] button.
     */
    private void doSendDataFile() {
        writeLog(getString(R.string.send_data_file), WRITE_LOG_IN);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.sendDataFile(mSelectPath, Dithering.DITHERING_ERRORDIFFUSION);
            msg = getString(R.string.send_data_file) + getString(R.string.msg_ok);
        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.send_data_file) + getString(R.string.msg_ng, ret);
        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.send_data_file), WRITE_LOG_OUT, ret);
    }


    /**
     * Implementation when pressing the [sendDataFile] button.
     *
     * @param alignment Alignment
     */
    protected void doSendDataFile(PrintAlignment alignment) {
        writeLog(getString(R.string.send_data_file), WRITE_LOG_IN);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.sendDataFile(mSelectPath, alignment, Dithering.DITHERING_ERRORDIFFUSION);
            msg = getString(R.string.send_data_file) + getString(R.string.msg_ok);
        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.send_data_file) + getString(R.string.msg_ng, ret);
        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.send_data_file), WRITE_LOG_OUT, ret);
    }


    /**
     * Implementation when pressing the [getPrinterResponse] button.
     *
     * @param id Response type constant
     */
    protected void doGetPrinterResponse(int id) {
        writeLog(getString(R.string.get_printer_response), WRITE_LOG_IN);

        if (id == PrinterManager.PRINTER_RESPONSE_KEY_CODE) {
            ArrayList<String> buf = new ArrayList<String>();

            int ret = 0;
            String msg;
            StringBuffer keyCode = new StringBuffer(256);
            try {
                mPrinterManager.getPrinterResponse(id, buf);
                int size = buf.size();
                for (int index = 0; index < size; index++) {
                    if (index != 0) {
                        keyCode.append(",");
                    }
                    keyCode.append(buf.get(index));
                }

                msg = getString(R.string.get_printer_response) + getString(R.string.msg_ok) + getString(R.string.msg_get_printer_response_key, keyCode.toString());
            } catch (PrinterException e) {
                ret = e.getErrorCode();
                msg = getString(R.string.get_printer_response) + getString(R.string.msg_ng, ret);
            }
            Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

            writeLog(getString(R.string.get_printer_response), WRITE_LOG_OUT, ret, getString(R.string.msg_get_printer_response_key, keyCode.toString()));

        } else {
            int[] buf = new int[1];
            if (id == PrinterManager.PRINTER_RESPONSE_REQUEST) {
                buf[0] = RESPONSE_REQUEST_CODE;
            }
            int ret = 0;
            String msg;
            try {
                mPrinterManager.getPrinterResponse(id, buf);
                msg = getString(R.string.get_printer_response) + getString(R.string.msg_ok) + getString(R.string.msg_get_printer_response, buf[0]);
            } catch (PrinterException e) {
                ret = e.getErrorCode();
                msg = getString(R.string.get_printer_response) + getString(R.string.msg_ng, ret);
            }
            Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

            writeLog(getString(R.string.get_printer_response), WRITE_LOG_OUT, ret, getString(R.string.msg_get_printer_response, buf[0]));
        }

    }


    /**
     * Implementation when pressing the [registerLogo] button.
     *
     * @param logoID Logo ID to register (key code)
     */
    protected void doRegisterLogoID1(int logoID) {
        writeLog(getString(R.string.register_logo), WRITE_LOG_IN, Integer.toString(logoID));

        int ret = 0;
        String msg;
        try {
            mPrinterManager.registerLogo(mSelectPath, logoID, Dithering.DITHERING_ERRORDIFFUSION);
            msg = getString(R.string.register_logo) + getString(R.string.msg_ok);
        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.register_logo) + getString(R.string.msg_ng, ret);
        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.register_logo), WRITE_LOG_OUT, ret);
    }


    /**
     * Implementation when pressing the [registerLogo] button.
     *
     * @param logoID Logo ID to register (key code)
     */
    protected void doRegisterLogoID2(String logoID) {
        writeLog(getString(R.string.register_logo), WRITE_LOG_IN, 0, logoID);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.registerLogo(mSelectPath, logoID, Dithering.DITHERING_ERRORDIFFUSION);
            msg = getString(R.string.register_logo) + getString(R.string.msg_ok);
        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.register_logo) + getString(R.string.msg_ng, ret);
        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.register_logo), WRITE_LOG_OUT, ret);
    }


    /**
     * Implementation when pressing the [printLogo] button.
     *
     * @param logoID Logo ID to print (key code)
     */
    protected void doPrintLogoID1(int logoID) {
        writeLog(getString(R.string.print_logo), WRITE_LOG_IN, 0, Integer.toString(logoID));

        int ret = 0;
        String msg;
        try {
            mPrinterManager.printLogo(logoID);
            msg = getString(R.string.print_logo) + getString(R.string.msg_ok);
        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.print_logo) + getString(R.string.msg_ng, ret);
        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.print_logo), WRITE_LOG_OUT, ret);
    }


    /**
     * Implementation when pressing the [printLogo] button.
     *
     * @param logoID Logo ID to print (key code)
     */
    protected void doPrintLogoID2(String logoID, PrintAlignment alignment) {
        writeLog(getString(R.string.print_logo), WRITE_LOG_IN, 0, logoID);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.printLogo(logoID, alignment);

            msg = getString(R.string.print_logo) + getString(R.string.msg_ok);
        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.print_logo) + getString(R.string.msg_ng, ret);
        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.print_logo), WRITE_LOG_OUT, ret);
    }


    /**
     * Implementation when pressing the [unregisterLogo] button.
     *
     * @param logoID Logo ID to delete (key code)
     */
    protected void doUnregisterLogoID1(int logoID) {
        writeLog(getString(R.string.unregister_logo), WRITE_LOG_IN, 0, Integer.toString(logoID));

        int ret = 0;
        String msg;
        try {
            mPrinterManager.unregisterLogo(logoID);
            msg = getString(R.string.unregister_logo) + getString(R.string.msg_ok);
        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.unregister_logo) + getString(R.string.msg_ng, ret);

        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.unregister_logo), WRITE_LOG_OUT, ret);
    }


    /**
     * Implementation when pressing the [unregisterLogo] button.
     *
     * @param logoID Logo ID to delete (key code)
     */
    protected void doUnregisterLogoID2(String logoID) {
        writeLog(getString(R.string.unregister_logo), WRITE_LOG_IN, logoID);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.unregisterLogo(logoID);
            msg = getString(R.string.unregister_logo) + getString(R.string.msg_ok);
        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.unregister_logo) + getString(R.string.msg_ng, ret);
        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.unregister_logo), WRITE_LOG_OUT, ret);
    }


    /**
     * Implementation when pressing the [registerStyleSheet] button.
     *
     * @param styleSheetNo Style sheet number to register
     */
    protected void doRegisterStyleSheet(int styleSheetNo) {
        writeLog(getString(R.string.register_style_sheet), WRITE_LOG_IN, 0, Integer.toString(styleSheetNo));

        int ret = 0;
        String msg;
        try {
            mPrinterManager.registerStyleSheet(mSelectPath, styleSheetNo);
            msg = getString(R.string.register_style_sheet) + getString(R.string.msg_ok);
        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.register_style_sheet) + getString(R.string.msg_ng, ret);
        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.register_style_sheet), WRITE_LOG_OUT, ret);
    }


    /**
     * Implementation when pressing the [unregisterStyleSheet] button.
     *
     * @param styleSheetNo Style sheet number to delete
     */
    protected void doUnregisterStyleSheet(int styleSheetNo) {
        writeLog(getString(R.string.unregister_style_sheet), WRITE_LOG_IN, Integer.toString(styleSheetNo));

        int ret = 0;
        String msg;
        try {
            mPrinterManager.unregisterStyleSheet(styleSheetNo);
            msg = getString(R.string.unregister_style_sheet) + getString(R.string.msg_ok);
        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.unregister_style_sheet) + getString(R.string.msg_ng, ret);
        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.unregister_style_sheet), WRITE_LOG_OUT, ret);
    }

    /**
     * Implementation when pressing the [printPageMode] button.
     */
    protected void doPrintPageMode() {
        writeLog(getString(R.string.print_page_mode), WRITE_LOG_IN);
        int ret = 0;
        String msg;
        String fileName = "TicketImage.jpg";
        String path = getApplicationContext().getExternalFilesDir(null).getAbsolutePath();

        try {
            InputStream in = getAssets().open(fileName);
            try {
                File file = new File(path, fileName);
                FileOutputStream out = new FileOutputStream(file.getPath());
                byte[] buf = new byte[1024];
                int len;
                while ((len = in.read(buf)) > 0) {
                    out.write(buf, 0, len);
                }
                out.close();

            } catch (IOException e) {

            }
            in.close();
        } catch (IOException e) {

        }

        try {
            mPrinterManager.enterPageMode();
            mPrinterManager.setPageModeArea(0, 0, 576, 355);
            mPrinterManager.printPageModeRectangle(0, 0, 575, 344, LineStyle.LINESTYLE_THIN);
            mPrinterManager.printPageModeRectangle(7, 7, 567, 336, LineStyle.LINESTYLE_THIN);
            mPrinterManager.printPageModeLine(404, 11, 404, 334, LineStyle.LINESTYLE_THIN);
            mPrinterManager.printPageModeText(21, 37, "NO.123456789");
            mPrinterManager.printPageModeText(212, 330, "Date 2020-01-01");
            mPrinterManager.printPageModeImageFile(
                    10,
                    212,
                    path + "/" + fileName,
                    Dithering.DITHERING_DISABLE);
            mPrinterManager.setPageModeArea(404, 0, 163, 345);
            mPrinterManager.setPageModeDirection(Direction.DIRECTION_BOTTOM_TO_TOP);
            mPrinterManager.printPageModeBarcode(
                    20,
                    132,
                    BarcodeSymbol.BARCODE_SYMBOL_CODE128,
                    new byte[]{0x67, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x68},
                    ModuleSize.BARCODE_MODULE_WIDTH_2,
                    80,
                    HriPosition.HRI_POSITION_ABOVE,
                    CharacterFont.FONT_A);
            mPrinterManager.printPageMode(CuttingMethod.CUT_PARTIAL);
            mPrinterManager.exitPageMode();
            msg = getString(R.string.print_page_mode) + getString(R.string.msg_ok);
        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.print_page_mode) + getString(R.string.msg_ng, ret);
        }

        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();
        writeLog(getString(R.string.print_page_mode), WRITE_LOG_OUT, ret);
    }


    /**
     * Implementation when pressing the [setBarcodeScannerListener] button.
     *
     * @param notationMethod Notation method
     */
    protected void doSetBarcodeScannerListener(String notationMethod) {
        writeLog(getString(R.string.set_barcode_scanner_listener), WRITE_LOG_IN, notationMethod);

        int ret = 0;
        String msg;
        try{
            if (notationMethod.equals("NOTATION_STRING")){
                mIsStringDisplay = true;
            }else {
                mIsStringDisplay = false;
            }
            mBarcodeScannerListener = MainActivity.this;
            mPrinterManager.setBarcodeScannerListener(mBarcodeScannerListener);
            msg = getString(R.string.set_barcode_scanner_listener) + getString(R.string.msg_ok);
        }catch(PrinterException e){
            ret = e.getErrorCode();
            msg = getString(R.string.set_barcode_scanner_listener) + getString(R.string.msg_ng, ret);
        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.set_barcode_scanner_listener), WRITE_LOG_OUT, ret);
    }


    /**
     * Implementation when pressing the [showTemplate] button.
     *
     * @param time_ms show template time
     */
    protected void doShowTemplate(int time_ms) {
        writeLog(getString(R.string.show_template), WRITE_LOG_IN, Integer.toString(time_ms));

        int ret = 0;
        String msg;
        try {
            mPrinterManager.showTemplate(time_ms);
            msg = getString(R.string.show_template) + getString(R.string.msg_ok);
        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.show_template) + getString(R.string.msg_ng, ret);
        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.show_template), WRITE_LOG_OUT, ret);
    }

    protected void doShowSlide(int slideID, int time_ms){
        writeLog(getString(R.string.show_slide), WRITE_LOG_IN);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.showSlide(slideID, time_ms);
            msg = getString(R.string.show_slide) + getString(R.string.msg_ok);

        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.show_slide) + getString(R.string.msg_ng, ret);

        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.show_slide), WRITE_LOG_OUT, ret);
    }

    protected void doExecuteMacro(int macroID, int repeatCount){
        writeLog(getString(R.string.execute_macro), WRITE_LOG_IN);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.executeMacro(macroID, repeatCount);
            msg = getString(R.string.execute_macro) + getString(R.string.msg_ok);

        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.execute_macro) + getString(R.string.msg_ng, ret);

        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.execute_macro), WRITE_LOG_OUT, ret);
    }

    protected void doTurnOnScreen(boolean isOn){
        writeLog(getString(R.string.turn_on_screen), WRITE_LOG_IN);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.turnOnScreen(isOn);
            msg = getString(R.string.turn_on_screen) + getString(R.string.msg_ok);

        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.turn_on_screen) + getString(R.string.msg_ng, ret);

        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.turn_on_screen), WRITE_LOG_OUT, ret);
    }

    protected void doSelectTemplate(int templateID, int slideID){
        writeLog(getString(R.string.select_template), WRITE_LOG_IN);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.selectTemplate(templateID, slideID);
            msg = getString(R.string.select_template) + getString(R.string.msg_ok);

        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.select_template) + getString(R.string.msg_ng, ret);

        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.select_template), WRITE_LOG_OUT, ret);
    }

    protected void doSetTemplateImageData(int mapID, int imageID){
        writeLog(getString(R.string.set_template_image_data), WRITE_LOG_IN);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.setTemplateImageData(mapID, imageID);
            msg = getString(R.string.set_template_image_data) + getString(R.string.msg_ok);

        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.set_template_image_data) + getString(R.string.msg_ng, ret);

        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.set_template_image_data), WRITE_LOG_OUT, ret);
    }

    protected void doSelectTemplateTextObject(int mapID){
        writeLog(getString(R.string.select_template_text_object), WRITE_LOG_IN);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.selectTemplateTextObject(mapID);
            msg = getString(R.string.select_template_text_object) + getString(R.string.msg_ok);

        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.select_template_text_object) + getString(R.string.msg_ng, ret);

        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.select_template_text_object), WRITE_LOG_OUT, ret);
    }

    protected void doSetTemplateTextAlignment(PrintAlignment alignment){
        writeLog(getString(R.string.set_template_text_alignment), WRITE_LOG_IN);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.setTemplateTextAlignment(alignment);
            msg = getString(R.string.set_template_text_alignment) + getString(R.string.msg_ok);

        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.set_template_text_alignment) + getString(R.string.msg_ng, ret);

        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.set_template_text_alignment), WRITE_LOG_OUT, ret);
    }

    protected void doSetTemplateTextData(String text){
        writeLog(getString(R.string.set_template_text_data), WRITE_LOG_IN);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.setTemplateTextData(text);
            msg = getString(R.string.set_template_text_data) + getString(R.string.msg_ok);

        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.set_template_text_data) + getString(R.string.msg_ng, ret);

        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.set_template_text_data), WRITE_LOG_OUT, ret);
    }

    protected void doSetTemplateTextLeftMargin(int margin){
        writeLog(getString(R.string.set_template_text_data), WRITE_LOG_IN);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.setTemplateTextLeftMargin(margin);
            msg = getString(R.string.set_template_text_left_margin) + getString(R.string.msg_ok);

        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.set_template_text_left_margin) + getString(R.string.msg_ng, ret);

        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.set_template_text_left_margin), WRITE_LOG_OUT, ret);
    }

    protected void doSetTemplateTextLineSpacing(int spacing){
        writeLog(getString(R.string.set_template_text_line_spacing), WRITE_LOG_IN);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.setTemplateTextLineSpacing(spacing);
            msg = getString(R.string.set_template_text_line_spacing) + getString(R.string.msg_ok);

        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.set_template_text_line_spacing) + getString(R.string.msg_ng, ret);

        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.set_template_text_line_spacing), WRITE_LOG_OUT, ret);
    }

    protected void doSetTemplateTextBold(CharacterBold bold){
        writeLog(getString(R.string.set_template_text_bold), WRITE_LOG_IN);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.setTemplateTextBold(bold);
            msg = getString(R.string.set_template_text_bold) + getString(R.string.msg_ok);

        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.set_template_text_bold) + getString(R.string.msg_ng, ret);

        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.set_template_text_bold), WRITE_LOG_OUT, ret);
    }

    protected void doSetTemplateTextUnderline(CharacterUnderline underline){
        writeLog(getString(R.string.set_template_text_underline), WRITE_LOG_IN);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.setTemplateTextUnderline(underline);
            msg = getString(R.string.set_template_text_underline) + getString(R.string.msg_ok);

        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.set_template_text_underline) + getString(R.string.msg_ng, ret);

        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.set_template_text_underline), WRITE_LOG_OUT, ret);
    }

    protected void doSetTemplateTextSize(CharacterScale scale){
        writeLog(getString(R.string.set_template_text_size), WRITE_LOG_IN);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.setTemplateTextSize(scale);
            msg = getString(R.string.set_template_text_size) + getString(R.string.msg_ok);

        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.set_template_text_size) + getString(R.string.msg_ng, ret);

        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.set_template_text_size), WRITE_LOG_OUT, ret);
    }

    protected void doSetTemplateTextFont(CharacterFont font){
        writeLog(getString(R.string.set_template_text_font), WRITE_LOG_IN);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.setTemplateTextFont(font);
            msg = getString(R.string.set_template_text_font) + getString(R.string.msg_ok);

        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.set_template_text_font) + getString(R.string.msg_ng, ret);

        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.set_template_text_font), WRITE_LOG_OUT, ret);
    }

    protected void doSetTemplateTextRegisteredFont(RegisteredFont font){
        writeLog(getString(R.string.set_template_text_registered_font), WRITE_LOG_IN);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.setTemplateTextRegisteredFont(font);
            msg = getString(R.string.set_template_text_registered_font) + getString(R.string.msg_ok);

        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.set_template_text_registered_font) + getString(R.string.msg_ng, ret);

        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.set_template_text_registered_font), WRITE_LOG_OUT, ret);
    }

    protected void doSetTemplateTextRightSpacing(int spacing){
        writeLog(getString(R.string.set_template_text_right_spacing), WRITE_LOG_IN);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.setTemplateTextRightSpacing(spacing);
            msg = getString(R.string.set_template_text_right_spacing) + getString(R.string.msg_ok);

        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.set_template_text_right_spacing) + getString(R.string.msg_ng, ret);

        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.set_template_text_right_spacing), WRITE_LOG_OUT, ret);
    }

    protected void doSetTemplateTextColor(int color){
        writeLog(getString(R.string.set_template_text_color), WRITE_LOG_IN);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.setTemplateTextColor(color);
            msg = getString(R.string.set_template_text_color) + getString(R.string.msg_ok);

        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.set_template_text_color) + getString(R.string.msg_ng, ret);

        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.set_template_text_color), WRITE_LOG_OUT, ret);
    }

    protected void doSetTemplateBarcodeData(int mapID, String text){
        writeLog(getString(R.string.set_template_barcode_data), WRITE_LOG_IN);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.setTemplateBarcodeData(mapID, text);
            msg = getString(R.string.set_template_barcode_data) + getString(R.string.msg_ok);

        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.set_template_barcode_data) + getString(R.string.msg_ng, ret);

        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.set_template_barcode_data), WRITE_LOG_OUT, ret);
    }

    protected void doSetTemplateQrcodeData(int mapID, ModuleSize moduleSize, ErrorCorrection errorCorrection, QrDataMode mode, QrQuietZone quietZone, String text){
        writeLog(getString(R.string.set_template_qrcode_data), WRITE_LOG_IN);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.setTemplateQrCodeData(mapID, moduleSize, errorCorrection, mode, quietZone, text);
            msg = getString(R.string.set_template_qrcode_data) + getString(R.string.msg_ok);

        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.set_template_qrcode_data) + getString(R.string.msg_ng, ret);

        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.set_template_qrcode_data), WRITE_LOG_OUT, ret);
    }

    protected void doRegisterTemplate(int templateID, String label){
        writeLog(getString(R.string.register_template), WRITE_LOG_IN);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.registerTemplate(templateID, label, mSelectPath);
            msg = getString(R.string.register_template) + getString(R.string.msg_ok);

        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.register_template) + getString(R.string.msg_ng, ret);

        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.register_template), WRITE_LOG_OUT, ret);
    }

    protected void doUnregisterTemplate(int templateID){
        writeLog(getString(R.string.unregister_template), WRITE_LOG_IN);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.unregisterTemplate(templateID);
            msg = getString(R.string.unregister_template) + getString(R.string.msg_ok);

        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.unregister_template) + getString(R.string.msg_ng, ret);

        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.unregister_template), WRITE_LOG_OUT, ret);
    }

    protected void doRegisterImageData(int imageID, String label){
        writeLog(getString(R.string.register_image_data), WRITE_LOG_IN);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.registerImageData(imageID, label, mSelectPath);
            msg = getString(R.string.register_image_data) + getString(R.string.msg_ok);

        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.register_image_data) + getString(R.string.msg_ng, ret);

        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.register_image_data), WRITE_LOG_OUT, ret);
    }

    protected void doUnregisterImageData(int imageID){
        writeLog(getString(R.string.unregister_image_data), WRITE_LOG_IN);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.unregisterImageData(imageID);
            msg = getString(R.string.unregister_image_data) + getString(R.string.msg_ok);

        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.unregister_image_data) + getString(R.string.msg_ng, ret);

        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.unregister_image_data), WRITE_LOG_OUT, ret);
    }

    protected void doRegisterSlideData(int slideID, String label){
        writeLog(getString(R.string.register_slide_data), WRITE_LOG_IN);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.registerSlideData(slideID, label, mSelectPath);
            msg = getString(R.string.register_slide_data) + getString(R.string.msg_ok);

        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.register_slide_data) + getString(R.string.msg_ng, ret);

        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.register_slide_data), WRITE_LOG_OUT, ret);
    }

    protected void doUnregisterSlideData(int slideID){
        writeLog(getString(R.string.unregister_slide_data), WRITE_LOG_IN);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.unregisterSlideData(slideID);
            msg = getString(R.string.unregister_slide_data) + getString(R.string.msg_ok);

        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.unregister_slide_data) + getString(R.string.msg_ng, ret);

        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.unregister_slide_data), WRITE_LOG_OUT, ret);
    }

    protected void doRegisterUserDefinedCharacter(){
        writeLog(getString(R.string.register_user_defined_character), WRITE_LOG_IN);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.registerUserDefinedCharacter(mSelectPath);
            msg = getString(R.string.register_user_defined_character) + getString(R.string.msg_ok);

        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.register_user_defined_character) + getString(R.string.msg_ng, ret);

        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.register_user_defined_character), WRITE_LOG_OUT, ret);
    }

    protected void doRegisterOptionFont(int startCode, int endCode, int width, int height){
        writeLog(getString(R.string.register_option_font), WRITE_LOG_IN);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.registerOptionFont(startCode, endCode, width, height, mSelectPath);
            msg = getString(R.string.register_option_font) + getString(R.string.msg_ok);

        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.register_option_font) + getString(R.string.msg_ng, ret);

        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.register_option_font), WRITE_LOG_OUT, ret);

    }

    protected void doControlMacroRegistration(int macroID, MacroRegistrationFunction control){
        writeLog(getString(R.string.control_macro_registration), WRITE_LOG_IN);

        int ret = 0;
        String msg;
        try {
            mPrinterManager.controlMacroRegistration(macroID, control);
            msg = getString(R.string.control_macro_registration) + getString(R.string.msg_ok);

        } catch (PrinterException e) {
            ret = e.getErrorCode();
            msg = getString(R.string.control_macro_registration) + getString(R.string.msg_ng, ret);

        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.control_macro_registration), WRITE_LOG_OUT, ret);

    }

    protected void doGetDisplayResponse(int responseID, int param){
        writeLog(getString(R.string.get_display_response), WRITE_LOG_IN);

        int ret = 0;
        String msg = new String();
        try{
            int[] buf;
            ArrayList<Integer> list;
            ArrayList<String> label;
            switch (responseID){
                case PrinterManager.DISPLAY_RESPONSE_REQUEST:
                    buf = new int[]{RESPONSE_REQUEST_CODE};
                    mPrinterManager.getDisplayResponse(responseID, param, buf);
                    msg = getString(R.string.get_display_response) + getString(R.string.msg_ok) + getString(R.string.msg_get_printer_response, buf[0]);
                    break;
                case PrinterManager.DISPLAY_RESPONSE_USER_AREA:
                    buf = new int[1];
                    mPrinterManager.getDisplayResponse(responseID, param, buf);
                    msg = getString(R.string.get_display_response) + getString(R.string.msg_ok) + getString(R.string.msg_get_printer_response, buf[0]);
                    break;
                case PrinterManager.DISPLAY_RESPONSE_TEMPLATE_ID_LIST:
                case PrinterManager.DISPLAY_RESPONSE_SLIDE_ID_LIST:
                case PrinterManager.DISPLAY_RESPONSE_IMAGE_ID_LIST:
                    list = new ArrayList<Integer>();
                    mPrinterManager.getDisplayResponse(responseID, param, list);
                    StringBuilder stringBuilder = new StringBuilder();
                    int size = list.size();
                    for (int index = 0; index < size; index++) {
                        if (index != 0) {
                            stringBuilder.append(",");
                        }
                        int num = list.get(index);
                        String str = String.valueOf(num);
                        stringBuilder.append(str);
                    }
                    msg = getString(R.string.get_printer_response) + getString(R.string.msg_ok) + getString(R.string.msg_get_printer_response_key, stringBuilder.toString());
                    break;
                case PrinterManager.DISPLAY_RESPONSE_TEMPLATE_LABEL:
                case PrinterManager.DISPLAY_RESPONSE_SLIDE_LABEL:
                case PrinterManager.DISPLAY_RESPONSE_IMAGE_LABEL:
                    label = new ArrayList<String>();
                    mPrinterManager.getDisplayResponse(responseID, param, label);
                    msg = getString(R.string.get_display_response) + getString(R.string.msg_ok) + getString(R.string.msg_get_display_response_label, label.get(0));
                    break;
                default:
            }
        }catch (PrinterException e){
            ret = e.getErrorCode();
            msg = getString(R.string.get_display_response) + getString(R.string.msg_ng, ret);
        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.get_display_response), WRITE_LOG_OUT, ret, msg);
    }

    protected void doPrintSmartLabelImageData(){
        writeLog(getString(R.string.print_smart_label_image_data), WRITE_LOG_IN);

        int ret = 0;
        String msg = new String();
        try{
            mPrinterManager.printSmartLabelImageData(mSmartLabelManager);
            msg = getString(R.string.print_smart_label_image_data) + getString(R.string.msg_ok);
        }catch (PrinterException e){
            ret = e.getErrorCode();
            msg = getString(R.string.print_smart_label_image_data) + getString(R.string.msg_ng, ret);
        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.print_smart_label_image_data), WRITE_LOG_OUT, ret);
    }

    protected void doSelectSmartLabelFile(){
        writeLog(getString(R.string.select_smart_label_file), WRITE_LOG_IN);

        int ret = 0;
        String msg = new String();
        try{
            mSmartLabelManager = new SmartLabelManager(getApplicationContext());
            mSmartLabelManager.selectSmartLabelFile(mSelectPath);
            msg = getString(R.string.select_smart_label_file) + getString(R.string.msg_ok);
        }catch (PrinterException e){
            ret = e.getErrorCode();
            msg = getString(R.string.select_smart_label_file) + getString(R.string.msg_ng, ret);
        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.select_smart_label_file), WRITE_LOG_OUT, ret);
    }

    protected void doReplaceSmartLabelTextData(int mapID, String text){
        writeLog(getString(R.string.replace_smart_label_text_data), WRITE_LOG_IN);

        int ret = 0;
        String msg = new String();
        try{
            mSmartLabelManager.replaceSmartLabelTextData(mapID, text);
            msg = getString(R.string.replace_smart_label_text_data) + getString(R.string.msg_ok);
        }catch (PrinterException e){
            ret = e.getErrorCode();
            msg = getString(R.string.replace_smart_label_text_data) + getString(R.string.msg_ng, ret);
        }catch (NullPointerException e){
            ret = PrinterException.ERROR_LABEL_FILE_NOT_SELECTED;
            msg = getString(R.string.replace_smart_label_text_data) + getString(R.string.msg_ng, ret);
        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.replace_smart_label_text_data), WRITE_LOG_OUT, ret);
    }

    protected void doReplaceSmartLabelBarcodeData(int mapID, String text){
        writeLog(getString(R.string.replace_smart_label_barcode_data), WRITE_LOG_IN);

        int ret = 0;
        String msg = new String();
        try{
            mSmartLabelManager.replaceSmartLabelBarcodeData(mapID, text);
            msg = getString(R.string.replace_smart_label_barcode_data) + getString(R.string.msg_ok);
        }catch (PrinterException e){
            ret = e.getErrorCode();
            msg = getString(R.string.replace_smart_label_barcode_data) + getString(R.string.msg_ng, ret);
        }catch (NullPointerException e){
            ret = PrinterException.ERROR_LABEL_FILE_NOT_SELECTED;
            msg = getString(R.string.replace_smart_label_barcode_data) + getString(R.string.msg_ng, ret);
        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.replace_smart_label_barcode_data), WRITE_LOG_OUT, ret);
    }

    protected void doReplaceSmartLabelImageData(int mapID){
        writeLog(getString(R.string.replace_smart_label_image_data), WRITE_LOG_IN);

        int ret = 0;
        String msg = new String();
        Bitmap image = null;
        try{
            InputStream stream = getApplicationContext().getContentResolver().openInputStream(Uri.parse(mSelectPath));
            image = BitmapFactory.decodeStream(new BufferedInputStream(stream));
            mSmartLabelManager.replaceSmartLabelImageData(mapID, image);
            msg = getString(R.string.replace_smart_label_image_data) + getString(R.string.msg_ok);
        }catch (PrinterException e){
            ret = e.getErrorCode();
            msg = getString(R.string.replace_smart_label_image_data) + getString(R.string.msg_ng, ret);
        }catch (FileNotFoundException e) {
            ret = PrinterException.ERROR_FILE_NOT_FOUND;
            msg = getString(R.string.replace_smart_label_image_data) + getString(R.string.msg_ng, ret);
        }catch (NullPointerException e){
            ret = PrinterException.ERROR_LABEL_FILE_NOT_SELECTED;
            msg = getString(R.string.replace_smart_label_image_data) + getString(R.string.msg_ng, ret);
        }
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();

        writeLog(getString(R.string.replace_smart_label_image_data), WRITE_LOG_OUT, ret);
    }


    /**
     * Display an alert dialog.
     *
     * @param dialogId Dialog Id
     */
    private void showAlertDialog(int dialogId) {
        AlertDialogFragment alertDialogFragment = new AlertDialogFragment();
        Bundle args = new Bundle();
        args.putInt(AlertDialogFragment.ID, dialogId);
        args.putInt(AlertDialogFragment.MODEL, mPrinterManager.getPrinterModel());
        alertDialogFragment.setArguments(args);
        FragmentTransaction ft = this.getFragmentManager().beginTransaction();
        ft.add(alertDialogFragment, AlertDialogFragment.DIALOG);
        ft.commitAllowingStateLoss();
    }

    /**
     * Implementation when file is selected.
     *
     * @param path     Selected file
     * @param dialogId Dialog Id
     */
    protected void onFileSelected(String path, int dialogId) {
        if (path == null) {
            //not selected
        } else {
            mSelectPath = path;

            switch (dialogId) {
                case DIALOG_SEND_DATA_FILE:
                    int printerModel = mPrinterManager.getPrinterModel();
                    if ((printerModel == PrinterManager.PRINTER_MODEL_DPU_S245)
                            || (printerModel == PrinterManager.PRINTER_MODEL_DPU_S445)) {
                        doSendDataFile();
                    } else {
                        showAlertDialog(DIALOG_SEND_DATA_FILE);
                    }
                    break;
                case DIALOG_REGISTER_LOGO_ID1:
                    showAlertDialog(DIALOG_REGISTER_LOGO_ID1);
                    break;
                case DIALOG_REGISTER_LOGO_ID2:
                    showAlertDialog(DIALOG_REGISTER_LOGO_ID2);
                    break;
                case DIALOG_REGISTER_STYLE_SHEET_NO:
                    showAlertDialog(DIALOG_REGISTER_STYLE_SHEET_NO);
                    break;
                case DIALOG_REGISTER_TEMPLATE:
                    showAlertDialog(DIALOG_REGISTER_TEMPLATE);
                    break;
                case DIALOG_REGISTER_IMAGE_DATA:
                    showAlertDialog(DIALOG_REGISTER_IMAGE_DATA);
                    break;
                case DIALOG_REGISTER_SLIDE_DATA:
                    showAlertDialog(DIALOG_REGISTER_SLIDE_DATA);
                    break;
                case DIALOG_REGISTER_USER_DEFINED_CHARACTER:
                    doRegisterUserDefinedCharacter();
                    break;
                case DIALOG_REGISTER_OPTION_FONT:
                    showAlertDialog(DIALOG_REGISTER_OPTION_FONT);
                    break;
                case DIALOG_SELECT_SMART_LABEL_FILE:
                    doSelectSmartLabelFile();
                    break;
                case DIALOG_REPLACE_SMART_LABEL_IMAGE_DATA:
                    showAlertDialog(DIALOG_REPLACE_SMART_LABEL_IMAGE_DATA);
                    break;
                default:
                    break;
            }
        }

    }

    /** After that, implementation of private methods */

    private byte[] asByteArray(String hex) {
        byte[] bytes = new byte[hex.length() / 2];

        try {
            for (int index = 0; index < bytes.length; index++) {
                String byteStr = hex.substring(index * 2, (index + 1) * 2);
                bytes[index] = (byte) Integer.parseInt(byteStr, 16);
            }
        } catch (IndexOutOfBoundsException e) {
        } catch (NumberFormatException e) {
        }

        return bytes;
    }

    private boolean checkBluetooth() {
        BluetoothAdapter bluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
        if (bluetoothAdapter == null) {
            showAlertDialog(DIALOG_BLUETOOTH_NO_SUPPORT);
            return false;
        }

        if (!bluetoothAdapter.isEnabled()) {
            Intent enableBluetoothIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
            startActivityForResult(enableBluetoothIntent, REQUEST_ENABLE_BLUETOOTH);
            return false;
        }
        return true;
    }

    private boolean checkWifi() {
        WifiManager wifiManager = (WifiManager) getApplicationContext().getSystemService(Context.WIFI_SERVICE);
        if (wifiManager == null) {
            return false;
        }

        if (wifiManager.getWifiState() != wifiManager.WIFI_STATE_ENABLED) {
            showAlertDialog(DIALOG_ENABLE_WIFI);
            return false;
        }
        return true;
    }

    protected void enableWifi() {
        WifiManager wifiManager = (WifiManager) getApplicationContext().getSystemService(Context.WIFI_SERVICE);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q) {
            Intent panelIntent = new Intent(Settings.Panel.ACTION_INTERNET_CONNECTIVITY);
            this.startActivityForResult(panelIntent,REQUEST_CODE_WIFI);
        }else if (wifiManager != null && wifiManager.setWifiEnabled(true)) {
            Toast.makeText(getApplicationContext(), R.string.enabled_wifi, Toast.LENGTH_LONG).show();
        } else {
            Toast.makeText(getApplicationContext(), R.string.faled_to_enable_wifi, Toast.LENGTH_LONG).show();
        }
    }

    private void setProperty() {
        if (mPrinterManager != null) {
            final SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(this);
            int sendTimeout = StringUtil.getInt(pref.getString(getString(R.string.key_send_timeout), getString(R.string.send_timeout_default)));
            mPrinterManager.setSendTimeout(sendTimeout);

            int receiveTimeout = StringUtil.getInt(pref.getString(getString(R.string.key_receive_timeout), getString(R.string.receive_timeout_default)));
            mPrinterManager.setReceiveTimeout(receiveTimeout);

            int socketKeepingTime = StringUtil.getInt(pref.getString(getString(R.string.key_socket_keeping_time), getString(R.string.socket_keeping_time_default)));
            mPrinterManager.setSocketKeepingTime(socketKeepingTime);

            int internationalCharacter;
            int codePage;
            if (Locale.JAPAN.equals(Locale.getDefault())) {
                internationalCharacter = StringUtil.getInt(pref.getString(getString(R.string.key_international_character), getString(R.string.international_character_default_jp_values)));
                codePage = StringUtil.getInt(pref.getString(getString(R.string.key_code_page), getString(R.string.code_page_default_jp_values)));
            } else {
                internationalCharacter = StringUtil.getInt(pref.getString(getString(R.string.key_international_character), getString(R.string.international_character_default_en_values)));
                codePage = StringUtil.getInt(pref.getString(getString(R.string.key_code_page), getString(R.string.code_page_default_en_values)));
            }
            mPrinterManager.setInternationalCharacter(internationalCharacter);
            mPrinterManager.setCodePage(codePage);
        }
    }

    private void writeLog(String command, boolean start) {
        writeLog(command, start, 0);
    }

    private void writeLog(String command, boolean start, String msg) {
        writeLog(command, start, 0, msg);
    }

    private void writeLog(String command, boolean start, int returnCode) {
        writeLog(command, start, returnCode, "");
    }

    private void writeLog(String command, boolean start, int returnCode, String msg) {
        EditText edtLog = (EditText) findViewById(R.id.edittext_log);
        if (edtLog != null) {
            StringBuffer buf = new StringBuffer(128);
            buf.append(getString(R.string.write_log_bracket, StringUtil.getDateString(getString(R.string.write_log_date))));

            buf.append(getString(R.string.write_log_command, command));
            if (start) {
                buf.append(getString(R.string.write_log_in));
            } else {
                buf.append(getString(R.string.write_log_out));
                if (returnCode != 0) {
                    buf.append(getString(R.string.write_log_result, returnCode));
                }
            }

            if (!StringUtil.isEmpty(msg)) {
                buf.append(getString(R.string.write_log_msg, msg));
            }

            buf.append(getString(R.string.write_log_end));

            edtLog.append(buf.toString());
        }
    }

    private void checkLocationRequest(){
        final LocationRequest locationRequest = LocationRequest.create();
        locationRequest.setPriority(LocationRequest.PRIORITY_HIGH_ACCURACY);
        LocationSettingsRequest.Builder builder = new LocationSettingsRequest.Builder().addLocationRequest(locationRequest);
        SettingsClient client = LocationServices.getSettingsClient(MainActivity.this);
        Task<LocationSettingsResponse> task = client.checkLocationSettings(builder.build());
        task.addOnCompleteListener(new OnCompleteListener<LocationSettingsResponse>() {
            @Override
            public void onComplete(@NonNull Task<LocationSettingsResponse> task) {
                try {
                    task.getResult(ApiException.class);
                    Intent intent = new Intent(MainActivity.this, DeviceActivity.class);
                    intent.putExtra(DeviceActivity.TYPE, mSelectPort);
                    startActivityForResult(intent, REQUEST_SELECT_DEVICE);
                } catch (ApiException exception) {
                    switch (exception.getStatusCode()) {
                        case LocationSettingsStatusCodes.RESOLUTION_REQUIRED:
                            try {
                                // Request for location information setting
                                ((ResolvableApiException) exception).startResolutionForResult(MainActivity.this, REQUEST_LOCATION_SETTING_RESOLUTION);
                                return;
                            } catch (IntentSender.SendIntentException e) {
                                // Ignore the error.
                            } catch (ClassCastException e) {
                                // Ignore, should be an impossible error.
                            }
                            break;
                        case LocationSettingsStatusCodes.SETTINGS_CHANGE_UNAVAILABLE:
                            break;
                    }
                }
            }
        });
    }

    /**
     * Display choose file dialog.
     *
     * @param dialogId Dialog Id
     */
    private void showOpenFileDialog(int dialogId) {
        Intent intent = new Intent(Intent.ACTION_OPEN_DOCUMENT);
        intent.addCategory(Intent.CATEGORY_OPENABLE);

        switch (dialogId) {
            case DIALOG_REGISTER_LOGO_ID1:
            case DIALOG_REGISTER_LOGO_ID2:
            case DIALOG_REGISTER_IMAGE_DATA:
            case DIALOG_REGISTER_SLIDE_DATA:
            case DIALOG_REPLACE_SMART_LABEL_IMAGE_DATA:
                intent.setType("image/*");
                break;
            case DIALOG_REGISTER_STYLE_SHEET_NO:
                intent.setType("text/css");
                break;
            case DIALOG_REGISTER_TEMPLATE:
                intent.setType("text/xml");
                break;
            case DIALOG_REGISTER_USER_DEFINED_CHARACTER:
            case DIALOG_REGISTER_OPTION_FONT:
            case DIALOG_SELECT_SMART_LABEL_FILE:
                intent.setType("application/octet-stream");
                break;
            case DIALOG_SEND_DATA_FILE:
            default:
                intent.setType("*/*");
                break;
        }

        startActivityForResult(intent, dialogId);
    }
}
